#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#$Id: SpecReply.py,v 1.1 2004/08/23 11:16:09 guijarro Exp $
"""SpecReply module

This module defines the SpecReply class
"""
import SpecEventsDispatcher as SpecEventsDispatcher

from pyspec.css_logger import log

REPLY_ID_LIMIT = 2**30
current_id = 0

def next_id():
    global current_id
    current_id = (current_id + 1) % REPLY_ID_LIMIT
    return current_id

class SpecReply(object):
    """SpecReply class

    Represent a reply received from a remote Spec server

    Signals:
    replyFromSpec(self) -- emitted on update
    """
    def __init__(self):
        """Constructor."""
        self.data = None
        self.error = False
        self.error_code = 0 #no error
        self.id = next_id()
        self.pending = True

    def update(self, data, error, error_code):
        """Emit the 'replyFromSpec' signal."""

        self.data = data
        self.error = error
        self.error_code = error_code

        self.pending = False
        SpecEventsDispatcher.emit(self, 'replyFromSpec', (self, ))

    def is_pending(self):
        return self.pending

    def get_data(self):
        """Return the value of the reply object (data field)."""
        return self.data



#******************************************************************************
#
#  %W%  %G% CSS
#
#  "pyspec" Release %R%
#
#  Copyright (c) 2013,2014,2015,2016,2017,2018,2020
#  by Certified Scientific Software.
#  All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software ("pyspec") and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  Neither the name of the copyright holder nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#     * The software is provided "as is", without warranty of any   *
#     * kind, express or implied, including but not limited to the  *
#     * warranties of merchantability, fitness for a particular     *
#     * purpose and noninfringement.  In no event shall the authors *
#     * or copyright holders be liable for any claim, damages or    *
#     * other liability, whether in an action of contract, tort     *
#     * or otherwise, arising from, out of or in connection with    *
#     * the software or the use of other dealings in the software.  *
#
#******************************************************************************


#
# File to load the Qt toolkit module
#   - it is possible to load qt4, qt5 or qtside with this module
#
#
# Choice method:
#
#   1.  command line flag
#       The following flags are recognided if provided in the command line
#         --pyside
#         --pyside2
#         --pyqt4
#         --pyqt5
#         --matplotlib
#         --qwt
#
#   2.  QT_API  and GRAPH_LIB environment variables
#         If no flag is provided, the module will check the QT_API
#         environment variable for one of the following values
#            'pyside', 'pyside2', 'pyqt4', 'pyqt5'
#
#         If no flag for graphics is provided, the module will check the GRAPH_API
#         environment variable for one of the following values
#            'matplotlib', 'qwt'
#
#         The environment variable setting is case-insensitive
#
#   3.  Try and fail
#         If no flag is provided and the QT_API and/or GRAPH_API environment 
#         variables are not set with any of the recognized values the module 
#         with try to import a possible compatible combination.
#

import sys
import os

from graphics_rc import g_rc

cmdline = False
varset = False

env_api = os.environ.get('QT_API', None)
env_graphics = os.environ.get('GRAPH_LIB', None)

if '--pyside' in sys.argv:
    g_rc.qt_variant = 'PySide'
    cmdline = True
elif '--pyside2' in sys.argv:
    g_rc.qt_variant = 'PySide2'
    cmdline = True
elif '--pyqt4' in sys.argv:
    g_rc.qt_variant = 'PyQt4'
    cmdline = True
elif '--pyqt5' in sys.argv:
    g_rc.qt_variant = 'PyQt5'
    cmdline = True
elif env_api is not None:
    if env_api.lower() == 'pyside':
        g_rc.qt_variant = 'PySide'
        varset = True
    elif env_api.lower() == 'pyside2':
        g_rc.qt_variant = 'PySide2'
        varset = True
    elif env_api.lower() == 'pyqt4':
        g_rc.qt_variant = 'PyQt4'
        varset = True
    elif env_api.lower() == 'pyqt5':
        g_rc.qt_variant = 'PyQt5'
        varset = True
    else:
        varset = None

if '--matplotlib' in sys.argv:
    g_rc.graph_variant = "matplotlib"
elif '--qwt' in sys.argv:
    g_rc.graph_variant = "qwt"
elif env_graphics is not None:
    if env_graphics.lower() == 'matplotlib':
        g_rc.graph_variant = "matplotlib"
    elif env_graphics.lower() == 'qwt':
        g_rc.graph_variant = "qwt"

user_selected = cmdline or varset

# decide whether importing qwt or matplotlib. default: matplotlib

def check_compatible():

    # returns 0 if incompatible

    if (not g_rc.qwt_imported) and (not g_rc.mpl_imported):
        return(False)

    elif g_rc.mpl_imported:
        if g_rc.qt_imported == False:
            return(False)
        elif g_rc.qt_variant in ["PySide", "PySide2"]:
            if g_rc.mpl_version_no < [1,1,0]:
                return(False)
        elif g_rc.qt_variant == "PyQt5":
            if g_rc.mpl_version_no < [1,4,1]:
                return(False)
        elif g_rc.qt_variant != "PyQt4":
            return(False)
    elif g_rc.qwt_imported and g_rc.qt_variant != "PyQt4":
        return(False)

    return(True)

def check():
    compat = check_compatible() and 1 or 0

    # if imports ok try to figure out a buggy python3/PyQt5 installation 
    if compat:
        from subprocess import Popen, PIPE

        if sys.version_info[0] == 2:
            py_exec = 'python2'
        else:
            py_exec = 'python3'

        from pyspec import _pyspec_dir
        inst_path = os.path.join(_pyspec_dir,'..') 
        shell_command  = "{} -c".format(py_exec)
        shell_command += " 'import sys;sys.path.append(\"{}\");".format(inst_path)
        shell_command += " from pyspec.graphics.QVariant import QApplication; app=QApplication([])'"
        proc = Popen(shell_command, shell=True, stdout=PIPE, stderr=PIPE)
        out, err = proc.communicate()
        if proc.returncode != 0:
            # if starting a qt application ends abnormally
            compat = -1

    return compat

def app_libraries():
    import pyspec
    if g_rc.mpl_imported:
         graph_string = "matplotlib %s" % g_rc.mpl_version
    elif g_rc.qwt_imported:
         graph_string = "qwt5" 
 
    qt_string = "%s %s" % (g_rc.qt_variant, ".".join(map(str, g_rc.qt_version)))
    py_vers = sys.version_info
    py_string = "%s.%s" % (py_vers[0], py_vers[1])
    pyspec_vers = pyspec.__version__

    return {"python": py_string, "qt": qt_string, 
            "graphics": graph_string, "pyspec": pyspec_vers}
    
def print_selection():
    if g_rc.mpl_imported:
        print("   Matplotlib selected / version %s (no=%s)" % (g_rc.mpl_version, g_rc.mpl_version_no))
    elif g_rc.qwt_imported:
        print("   Qwt selected ")
    else:
        print("   No graphics library found")

    if not g_rc.qt_imported:
        print("   QtVariant is %s (cannot import)" % g_rc.qt_variant)
    else:
        print("   QtVariant is %s / version %s" % (g_rc.qt_variant, ".".join(map(str,g_rc.qt_version))))

    if check_compatible():
        print("\nCOMPATIBLE\n")
    else:
        print("\nINCOMPATIBLE\n")

    print("To change the library selection you may want to use:")
    print("    -    command line options: ['--matplotlib','--qwt','--pyqt4','--pyqt5','--pyside','--pyside2']")
    print("    -    QT_API env. variable: ['pyqt4','pyside','pyside2', 'pyqt5']")
    print("    - GRAPH_LIB env. variable: ['matplotlib','qwt']")

def debug_trace(obj):
    '''Set a tracepoint in the Python debugger that works with Qt'''

    import pdb

    if g_rc.qt_variant in ['PyQt4','PyQt5']:
       pyqtRemoveInputHook()

    pdb.set_trace()
 
#
#  MAIN importing.  
#    It cannot be done in a function as we want to do  
#         from <module> import *
#    not allowed inside a function
#

  # 
  # IMPORT Graphics
  # 
graphics_failed = False

  # matplotlib if selected by user
if g_rc.graph_variant == 'matplotlib':
    try:
        #from matplotlib import rcParams
        g_rc.mpl_available = True
    except:
        g_rc.mpl_available = False
        
    if not g_rc.mpl_available:
        graphics_failed = True

  # qwt if selected by user
if not graphics_failed and g_rc.graph_variant == 'qwt':
    if user_selected and g_rc.qt_variant != 'PyQt4':
        graphics_failed = True
    else:
        from qwt_import import *

    if not g_rc.qwt_imported:
        graphics_failed = True

  # matplotlib as default choice
if not graphics_failed and not g_rc.mpl_available and not g_rc.qwt_imported:
    try:
        #from matplotlib import rcParams
        g_rc.mpl_available = True
    except:
        g_rc.mpl_available = False
        

  # qwt as last choice
if not graphics_failed and not g_rc.mpl_available and not g_rc.qwt_imported:
    from qwt_import import *

if not g_rc.mpl_available and not g_rc.qwt_imported:
    graphics_failed = True

if graphics_failed:
    print("Cannot find a supported graphical library.")

# if qwt has been imported only pyqt4 is possible
if g_rc.qwt_imported:
    if g_rc.qt_variant is None or g_rc.qt_variant == "PyQt4":
        from PyQt4_import import * 
elif g_rc.mpl_available:
    # first check what to do if there is a user selection (cmdline or env-variable)
    if g_rc.qt_variant == "PyQt5": 
        from PyQt5_import import *
    elif g_rc.qt_variant == "PySide": 
        from PySide_import import *
    elif g_rc.qt_variant == "PySide2": 
        from PySide2_import import *
    elif g_rc.qt_variant == "PyQt4": 
        from PyQt4_import import * 
 
    if not g_rc.qt_imported: 
        if user_selected:
             print("Could not import selected Qt toolkit %s " % g_rc.qt_variant)
        else:
             # if there is still no qt try pyqt5 - pyside - pyqt4 in that order
            from PyQt5_import import *
            if not g_rc.qt_imported:
                from PySide_import import *
                if not g_rc.qt_imported:
                    from PySide2_import import *
                    if not g_rc.qt_imported:
                        from PyQt4_import import * 

    if g_rc.mpl_available:
        # now really import matplotlib with g_rc.qt_imported known
        from matplotlib_import import *
else:
    # if there is no graphics library imported. do not even try
    pass

if not g_rc.qt_imported:
    if not user_selected:
        print("Cannot find a supported Qt toolkit library.")
    else:
        print("Cannot import selected Qt toolkit library %s."  % g_rc.qt_variant)
    
if __name__ == '__main__':
   # If run standalone print selection
   if "check" in sys.argv:
       print( check() )
   else:
       print_selection()
else:
    if (not check_compatible()):
        print_selection()
        print( """
{progname} needs graphical libraries installed in your system.

{progname} can run in the following environment:
   - matplotlib version 1.4.1 or later together with PyQt5
   - matplotlib version 1.1 or later together with PySide 
   - matplotlib version 0.99 or later together with PyQt4 
   - pyqwt5 with pyqt4 

No compatible installation was found.

Make sure compatible graphical libraries are installed in your system.
""").format( progname=os.path.basename(sys.argv[0]) )
        raise ImportError




import h5py

class H5Writer(object):
    def __init__(self):
        self._ofd = None
   
    def h5_open(self, filename, metadata=None, overwrite=False, append=True):
        pass
     
    def h5_save(self, data, filename, metadata=None):
        with self.h5_open(filename) as _ofd:
             _ofd.create_dataset("/entry/data/data")



#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#$Id: SpecMotor.py,v 1.6 2005/02/08 13:17:21 guijarro Exp $
"""SpecMotor module

This module defines the classes for motor objects

Classes:
SpecMotor -- class representing a motor in Spec
SpecMotorA -- class representing a motor in Spec, to be used with a GUI
"""

import math

from pyspec.css_logger import log

from SpecWaitObject import SpecWaitObject
from SpecChannel import SpecChannel
from SpecEventsDispatcher import UPDATEVALUE, FIREEVENT, callableObjectRef

(NOTINITIALIZED, UNUSABLE, READY, MOVESTARTED, MOVING, ONLIMIT) = (0,1,2,3,4,5)
(NOLIMIT, LOWLIMIT, HIGHLIMIT) = (0,2,4)

class SpecMotor(object):
    def __init__(self, conn, motormne, callbacks={}, timeout=None):


        self.motormne = motormne
        self._conn = conn

        if None in (motormne, conn):
            self._conn_ok = False
            return

        self.state = NOTINITIALIZED

        self.limit = NOLIMIT
        self.limits = (None, None)

        self.prev_position = None

        # the callbacks listed below can be set directly using the 'callbacks' keyword argument ;
        # when the event occurs, the corresponding callback will be called automatically
        self._callbacks = {
            'connected': None,
            'disconnected': None,
            'motorLimitsChanged': None,
            'motorPositionChanged': None,
            'motorStateChanged': None
        }

        # check that is a valid connection
        try:
            self.specname = self._conn.get_specname()
            self._conn_ok = True
        except Exception as e:
            import traceback
            log.log(2, "SpecMotor (%s) cannot get a valid spec connection" % motormne)
            log.log(2, traceback.format_exc())
            self._conn_ok = False
            return

        self.motormne = motormne
        self.chan_prefix = "motor/%s/%%s" % motormne

        self._conn.connect_event('connected', self.spec_connected)
        self._conn.connect_event('disconnected', self.spec_disconnected)

        if self._conn.is_connected():
            self.spec_connected()

        for cb_name in iter(self._callbacks.keys()):
            if callable(callbacks.get(cb_name)):
                self._callbacks[cb_name] = callableObjectRef(callbacks[cb_name])

    def update(self):
        self._conn.update()

    def spec_connected(self):
        pass

    def spec_disconnected(self):
        pass

    def read(self, chan_name):
        if not self._conn_ok:
            return
        return self._conn.read_channel(self.chan_prefix % chan_name)

    def write(self, chan_name, value):
        if not self._conn_ok:
            return
        self._conn.write_channel(self.chan_prefix % chan_name, value)
 
    @property
    def moving(self):
        done = self.read('move_done')
        return (not done)

    def wait(self, chan_name, done_value):
        ch = self.chan_prefix % 'move_done'
        w = SpecWaitObject(self._conn)
        w.wait_update(ch, done_value = done_value) 

    def move(self, target_position):
        """Move the motor

        Block until the move is finished

        Arguments:
        absolutePosition -- position where to move the motor to
        """
        try:
            target = float(target_position)
        except ValueError:
            log.log(1, "Cannot move %s: position '%s' is not a number" % \
                    self.motormne, target_position)
            return

        self.start_move(target)
        self.wait_move_done()
    mv = move

    def move_relative(self, inc_position):
        try:
            target = float(inc_position)
            target = self.get_position() + inc_position
        except ValueError:
            log.log(1, "Cannot move relative %s: position '%s' is not a number" % \
                    self.motormne, inc_position)
            return

        self.start_move(target)
        self.wait_move_done()
    mvr = move

    def start_move(self, target_position):
        self.write('start_one', target_position)

    def wait_move_done(self):
        self.wait('move_done', 0)

    def stop(self):
        """Stop the current motor

        Send an 'abort' message to the remote Spec
        """
        self._conn.abort()

    def get_position(self):
        """Return the current absolute position for the motor."""
        return self.read('position')

    @property
    def position(self):
        return self.get_position()

    def set_offset(self, offset):
        """Set the motor offset value"""
        self.write('offset', offset)

    def get_offset(self):
        return self.read('offset')

    def get_sign(self):
        return self.read('sign')

    def get_dial_position(self):
        return self.read('dial_position')

    def get_limits(self):
        """Return a (low limit, high limit) tuple in user units."""
        low_lim = self.read('low_limit')
        high_lim = self.read('high_limit')
        lims = [lim * self.get_sign() + self.get_offset() for lim in (low_lim, high_lim)]
        return (min(lims), max(lims))

    def unusable(self):
        return self.read('unusable')

    def low_limit_hit(self):
        return self.read('low_lim_hit')

    def high_limit_hit(self):
        return self.read('high_lim_hit')

    # backwards compatibility
    moveRelative = move_relative
    getPosition = get_position
    getDialPosition = get_dial_position
    getLimits = get_limits
    getSign = get_limits
    getOffset = get_offset
    setOffset = set_offset
    lowLimitHit = low_limit_hit
    highLimitHit = high_limit_hit
    getParameter = read
    setParameter = write

class SpecMotorA(SpecMotor):
    """SpecMotorA (asynchronous) class"""

    def register(self, channame, cb, mode=UPDATEVALUE):
        self._conn.registerChannel(self.chan_prefix % channame,  cb, dispatchMode=mode)

    def connected(self):
        """Callback triggered by a 'connected' event from Spec

        To be extended by derivated classes.
        """
        pass

    def spec_connected(self):
        """Private callback triggered by a 'connected' event from Spec."""
        #
        # register channels
        #
        self.register('position', self.__motorPositionChanged, mode = FIREEVENT)
        self.register('move_done', self.motorMoveDone, mode = FIREEVENT)
        self.register('sync_check', self.__syncQuestion)
        self.register('unusable', self.__motorUnusable)
        self.register('high_lim_hit', self.__motorLimitHit)
        self.register('low_lim_hit', self.__motorLimitHit)

        self.register('low_limit', self._motorLimitsChanged)
        self.register('high_limit', self._motorLimitsChanged)
        self.register('offset', self.motorOffsetChanged)
        self.register('sign', self.signChanged)

        self.update()
 
        try: 
            if self._callbacks.get("connected"):
                cb = self._callbacks["connected"]()
                if cb is not None:
                    cb()
        finally:
            self.connected()

    def disconnected(self):
        """Callback triggered by a 'disconnected' event from Spec

        To be extended by derivated classes.
        """
        pass

    def spec_disconnected(self):
        """Private callback triggered by a 'disconnected' event from Spec

        Put the motor in NOTINITIALIZED state.
        """
        self.__changeMotorState(NOTINITIALIZED)

        try:
          if self._callbacks.get("disconnected"):
            cb = self._callbacks["disconnected"]()
            if cb is not None:
              cb()
        finally:
            self.disconnected()

    def signChanged(self, sign):
        self._motorLimitsChanged()

    def motorOffsetChanged(self, offset):
        self._motorLimitsChanged()

    def _motorLimitsChanged(self):
        try:
          if self._callbacks.get("motorLimitsChanged"):
            cb = self._callbacks["motorLimitsChanged"]()
            if cb is not None:
               cb()
        finally:
           self.motorLimitsChanged()

    def motorLimitsChanged(self):
        """Callback triggered by a 'low_limit' or a 'high_limit' channel update,
        or when the sign or offset for motor changes

        To be extended by derivated classes.
        """
        pass

    def motorMoveDone(self, channelValue):
        """Callback triggered when motor starts or stops moving

        Change the motor state accordingly.

        Arguments:
        channelValue -- value of the channel
        """
        if channelValue:
            self.__changeMotorState(MOVING)
        elif self.state in (MOVING, MOVESTARTED,NOTINITIALIZED):
            self.__changeMotorState(READY)

    def __motorLimitHit(self, channelValue, channelName):
        """Private callback triggered by a 'low_lim_hit' or a 'high_lim_hit' channel update

        Update the motor state accordingly.

        Arguments:
        channelValue -- value of the channel
        channelName -- name of the channel (either 'low_lim_hit' or 'high_lim_hit')
        """
        if channelValue:
            if channelName.endswith('low_lim_hit'):
                self.limit = self.limit | LOWLIMIT
                self.__changeMotorState(ONLIMIT)
            else:
                self.limit = self.limit | HIGHLIMIT
                self.__changeMotorState(ONLIMIT)

    def __motorPositionChanged(self, absolutePosition):
        if self.prev_position is None:
            self.prev_position = absolutePosition
        else:
            if math.fabs(absolutePosition - self.prev_position) > 1E-6:
                self.prev_position = absolutePosition
            else:
                return
        try:
            if self._callbacks.get("motorPositionChanged"):
                cb = self._callbacks["motorPositionChanged"]()
                if cb is not None:
                   cb(absolutePosition)
        finally:
            self.motorPositionChanged(absolutePosition)

    def motorPositionChanged(self, absolutePosition):
        """Callback triggered by a position channel update

        To be extended by derivated classes.

        Arguments:
        absolutePosition -- motor absolute position
        """
        pass

    def __syncQuestion(self, channelValue):
        """Callback triggered by a 'sync_check' channel update

        Call the self.syncQuestionAnswer method and reply to the sync question.

        Arguments:
        channelValue -- value of the channel
        """
        if type(channelValue) == type(''):
            steps = channelValue.split()
            specSteps = steps[0]
            controllerSteps = steps[1]

            a = self.syncQuestionAnswer(specSteps, controllerSteps)

            if a is not None:
                self._conn.write_channel(self.chan_prefix % 'sync_check', a)

    def syncQuestionAnswer(self, specSteps, controllerSteps):
        """Answer to the sync. question

        Return either '1' (YES) or '0' (NO)

        Arguments:
        specSteps -- steps measured by Spec
        controllerSteps -- steps indicated by the controller
        """
        pass

    def __motorUnusable(self, unusable):
        """Private callback triggered by a 'unusable' channel update

        Update the motor state accordingly

        Arguments:
        unusable -- value of the channel
        """
        if unusable:
            self.__changeMotorState(UNUSABLE)
        else:
            self.__changeMotorState(READY)

    def __changeMotorState(self, state):
        """Private method for changing the SpecMotor object's internal state

        Arguments:
        state -- the motor state
        """
        self.state = state

        try:
            if self._callbacks.get("motorStateChanged"):
                cb = self._callbacks["motorStateChanged"]()
                if cb is not None:
                    cb(state)
        finally:
            self.motorStateChanged(state)

    def motorStateChanged(self, state):
        """Callback to take into account a motor state update

        To be extended by derivated classes

        Arguments:
        state -- the motor state
        """
        pass

    def move(self, target_position):
        self.__changeMotorState(MOVESTARTED)
        super(SpecMotorA, self).move(target_position)

    def move_relative(self, inc_position):
        self.__changeMotorState(MOVESTARTED)
        super(SpecMotorA, self).move_relative(inc_position)

    def wait_move_done(self):
        pass

    def getState(self):
        """Return the current motor state."""
        return self.state



#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#$id: SpecConnection.py,v 1.11 2005/12/09 10:32:24 guijarro Exp $
"""SpecConnection module

Low-level module for communicating with a
remove Spec server

Classes :
SpecClientNotConnectedError -- exception class
SpecConnection
SpecConnectionDispatcher
"""

__author__ = 'Matias Guijarro'
__version__ = '1.0'

import asyncore
import socket
import string
import traceback
import sys
import time

from pyspec.css_logger import log
from pyspec.utils import is_python3, is_remote_host

from SpecEventsDispatcher import UPDATEVALUE, FIREEVENT

from SpecClientError import SpecClientNotConnectedError, \
        SpecClientVersionError, SpecClientProtocolError, \
        SpecClientTimeoutError

import SpecEventsDispatcher 

import SpecChannel 
import SpecMessage
import SpecReply

from SpecCommand import SpecCommand
from pyspec.client.SpecMotor import SpecMotor, SpecMotorA

import spec_updater

try:
    import spec_shm
    shm_available = True
except Exception as e:
    shm_available = False

asyncore.dispatcher.ac_in_buffer_size = 32768 #32 ko input buffer

(DISCONNECTED, PORTSCANNING, WAITINGFORHELLO, CONNECTED) = (1,2,3,4)
(MIN_PORT, MAX_PORT) = (6510, 6530)

# timing
WAIT_HELLO_TIMEOUT = 5  # in seconds. timeout for hello reply
UPDATER_TIME = 10 # in millisecs.  update time for spec_updater

DEFAULT_REPLY_TIMEOUT = 1 # in seconds. 

def _spec_connection(_class):
    """ decorator to provide connection parameter singletons """
    _instances = {}

    def get_only_one(spec_app):
        """ creating or just return the one and only class instance.
            The singleton depends on the parameters used in __init__ """

        t = spec_app.split(":")
        if len(t) == 2:
            host, specname = t
        else:
            host = "localhost"
            specname = spec_app

        if _class.class_name == "QSpecConnection":
            thread_update = False
        else:
            thread_update = True

        key = (host, specname, thread_update)
        if key not in _instances:
            _instances[key] = _SpecConnection(host, specname, thread_update)

        return _instances[key]

    return get_only_one

@_spec_connection
class SpecConnection(object):
    """ 
    If using this class events are automatically propagated
    from updating thread. 
    Not thread safe. 
    Alternative: QSpecConnection
    """
    class_name = "SpecConnection"

@_spec_connection
class QSpecConnection(object):
    """ 
    If using this class program must take care of calling conn.update_events() 
    for events to propagate 

    create class with argument:  "host:specname"
    """
    class_name = "QSpecConnection"

class _SpecConnection(asyncore.dispatcher):
    """SpecConnection class

    Signals:
    connected() -- emitted when the required Spec version gets connected
    disconnected() -- emitted when the required Spec version gets disconnected
    replyFromSpec(reply id, SpecReply object) -- emitted when a reply comes from the remote Spec
    error(error code) -- emitted when an error event is received from the remote Spec
    """
    def __init__(self, host, specname, thread_update):
        """Constructor

        Arguments:
        spec_app -- a 'host:port' string
        """
        asyncore.dispatcher.__init__(self)

        self.updater = None
        log.log(2, "creating _SpecConnection. thread_update=%s" % thread_update)
        self.thread_update = thread_update

        self.socket_connected = False  # a socket is opened

        # state 
        self.state = DISCONNECTED 
        self.valid_socket = False

        self.scanports = False
        self.specname = ''

        self.message = None

        self.received_strings = []
        self.output_strings = []
        self.sendq = []

        self.server_version = None

        self.aliasedChannels = {}
        self.reg_channels = {}
        self.reg_replies = {}

        self.simulation_mode = False

        # some shortcuts
        self.macro = self.send_msg_cmd_with_return
        self.macro_noret = self.send_msg_cmd
        self.abort = self.send_msg_abort

        self.host = host
        self.specname = specname

        try:
            self.port = int(self.specname)
            self.specname = None
        except:
            self.port = None
            self.scanports = True

        #
        # register 'service' channels
        #
        self.registerChannel('error', self.error, dispatchMode = FIREEVENT)
        self.registerChannel('status/simulate', self.simulationStatusChanged)

        self.run()

    def get_host(self):
        return self.host

    def get_specname(self):
        return self.specname

    def is_remote(self):
        return is_remote_host(self.host)

    def __str__(self):
        return '<spec connection: %s (@%s:%s)>' % (self.specname, self.host, self.port)

    def set_socket(self, s):
        self.valid_socket = True
        asyncore.dispatcher.set_socket(self, s)

    # update thread handling
    def is_running(self):
        if self.updater is None:
            return False
        return self.updater.is_running()

    def run(self):
        if self.is_running():
           return

        # start a thread for automatic update
        self.updater = spec_updater.spec_updater(method=spec_updater.THREAD,
                update_time=UPDATER_TIME,
                update_func=self._update)

        self.updater.start()

    def _update(self,timeout=0.01):
        try:
            self.check_connection()
            if asyncore.socket_map:
                asyncore.loop(timeout=0.01, count=1)

            if self.thread_update:
                self.update_events()

        except Exception as e:
            import traceback
            log.log(2, traceback.format_exc())

    def close_connection(self):
        if self.updater is not None:
            self.updater.stop()
        self.close()

    def __del__(self):
        self.close_connection()

    # END update thread handling

    def wait_connected(self, timeout=2):
        if not self.updater.is_running():
            self.updater.run()

        start_waiting = time.time()

        while self.state in (DISCONNECTED, WAITINGFORHELLO):
            time.sleep(0.02)
            if time.time() - start_waiting > timeout:
                raise SpecClientTimeoutError

    def wait_channel_is(self, channel, done_value, timeout):
        pass

    def check_connection(self):
        """Establish a connection to Spec

        If the connection is already established, do nothing.
        Otherwise, create a socket object and try to connect.
        If we are in port scanning mode, try to connect using
        a port defined in the range from MIN_PORT to MAX_PORT
        """
        if not self.socket_connected:
            if self.scanports:
                if self.port is None or self.port > MAX_PORT:
                    self.port = MIN_PORT
                else:
                    self.port += 1

            while not self.scanports or self.port < MAX_PORT:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.settimeout(0.2)
                try:
                    if s.connect_ex( (self.host, self.port) ) == 0:
                        self.set_socket(s)
                        self.handle_connect()
                        break
                except socket.error:
                    pass # exception could be 'host not found' for example, we ignore it

                if self.scanports:
                    self.port += 1 
                else:
                    break
        elif self.state == WAITINGFORHELLO:
            if (time.time() - self.waiting_hello_started) > WAIT_HELLO_TIMEOUT:
                log.log(2, "socket connected but no response to hello message.forget this socket")
                self.handle_close()

    def register(self, chan_name, receiver_slot, registrationFlag = SpecChannel.DOREG, dispatchMode = UPDATEVALUE):
        """Register a channel

        Tell the remote Spec we are interested in receiving channel update events.
        If the channel is not already registered, create a new SpecChannel object,
        and connect the channel 'valueChanged' signal to the receiver slot. If the
        channel is already registered, simply add a connection to the receiver
        slot.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        receiver_slot -- any callable object in Python

        Keywords arguments:
        registrationFlag -- internal flag
        dispatchMode -- can be SpecEventsDispatcher.UPDATEVALUE (default) or SpecEventsDispatcher.FIREEVENT,
        depending on how the receiver slot will be called. UPDATEVALUE means we don't mind skipping some
        channel update events as long as we got the last one (for example, a motor position). FIREEVENT means
        we want to call the receiver slot for every event.
        """
        if dispatchMode is None:
            return

        chan_name = str(chan_name)

        try:
            if not chan_name in self.reg_channels:
                channel = SpecChannel.SpecChannel(self, chan_name, registrationFlag)

                self.reg_channels[chan_name] = channel

                if channel.spec_chan_name != chan_name:  # for assoc array elements
                    channel.registered = True
                    def valueChanged(value, chan_name=chan_name):
                        channel = self.reg_channels[chan_name]
                        channel.update(value) 
                    self.aliasedChannels[chan_name]=valueChanged
                    self.registerChannel(channel.spec_chan_name, valueChanged, registrationFlag, dispatchMode)
            else:
                channel = self.reg_channels[chan_name]
  
            SpecEventsDispatcher.connect(channel, 'valueChanged', receiver_slot, dispatchMode)
  
            channelValue = self.reg_channels[channel.spec_chan_name].value
            if channelValue is not None:
                # we received a value, so emit an update signal
                channel.update(channelValue,force=True)
        except Exception:
            traceback.print_exc()

    registerChannel = register

    def unregister(self, chan_name, receiver_slot=None):
        """Unregister a channel

        Arguments:
        chan_name -- a string representing the channel to unregister, i.e. 'var/toto'
        """
        chan_name = str(chan_name)

        if chan_name in self.reg_channels:
            channel = self.reg_channels[chan_name] 
            if receiver_slot: 
                SpecEventsDispatcher.disconnect(channel, 'valueChanged', receiver_slot)
            else:
                self.reg_channels[chan_name].unregister()
                del self.reg_channels[chan_name]

    unregisterChannel = unregister

    def run_cmd(self, cmd, timeout=2):
        cmd = SpecCommand(self, cmd, timeout=timeout)
        return cmd()

    def read_channel(self, chan_name):
        p = chan_name.split("/")
        if len(p) == 1:
            chan_name = "var/%s" % chan_name

        chan = self.get_channel(chan_name)
        return chan.read()

    def write_channel(self, chan_name,value):
        p = chan_name.split("/")
        if len(p) == 1:
            chan_name = "var/%s" % chan_name
        chan = self.get_channel(chan_name)
        return chan.write(value)

    def get_position(self, mnemonic):
        chan_name = "motor/%s/position" % mnemonic
        chan = self.get_channel(chan_name)
        return chan.read()

    def get_version(self):
        return self.get("var/VERSION")

    def get_name(self):
        return self.get("var/SPEC")

    def get_motor(self, mne):
        motor = SpecMotorA(self, mne)
        return motor

    def get_motors(self):
        macro = """local md[]; 
            for (i=0; i<MOTORS; i++) { 
                 md[motor_mne(i)]=motor_name(i) }; 
            return md"""

        return self.run_cmd(macro)

    def get_positions(self):
        macro = """ local mpos[]
           for (i=0;i<MOTORS;i++) {
              mpos[motor_mne(i)]=A[i]
           }
        return mpos"""
        positions = self.run_cmd(macro)
        for ky, val in positions.items():
            positions[ky] = float(val)
        return positions

    @property
    def spec_version(self):
        return self.get_version()

    @property
    def name(self):
        return self.get_name()

    def __getattr__(self, name):
        if name in self.get_motors().keys():
            return self.get_motor(name)
        else:
            raise AttributeError(name)

    def get_channel(self, chan_name):
        """Return a channel object

        If the required channel is already registered, return it.
        Otherwise, return a new 'temporary' unregistered SpecChannel object ;
        reference should be kept in the caller or the object will get dereferenced.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        """
        if chan_name not in self.reg_channels:
            # return a newly created temporary SpecChannel object, without registering
            return SpecChannel.SpecChannel(self, chan_name, SpecChannel.DONTREG)

        return self.reg_channels[chan_name]

    getChannel = get_channel


    def error(self, error):
        """Emit the 'error' signal when the remote Spec version signals an error."""
        log.error('Error from Spec: %s', error)
        SpecEventsDispatcher.emit(self, 'error', (error, ))

    def simulationStatusChanged(self, sim_state):
        self.simulation_mode = sim_state
        SpecEventsDispatcher.emit(self, 'simulation', (sim_state,))

    def is_simulation(self):
        return self.simulation_mode

    def is_connected(self):
        """Return True if the remote Spec version is connected."""
        return self.state == CONNECTED

    isSpecConnected = is_connected

    def spec_connected(self):
        """Emit the 'connected' signal when the remote Spec version is connected."""
        old_state = self.state
        self.state = CONNECTED
        if old_state != CONNECTED:
            log.log(1,'Connected to %s:%s' % (self.host, self.specname or self.port))
            SpecEventsDispatcher.emit(self, 'connected', ())

    def spec_disconnected(self):
        """Emit the 'disconnected' signal when the remote Spec version is disconnected."""
        # SpecEventsDispatcher.dispatch()
        old_state = self.state
        self.state = DISCONNECTED
        if old_state == CONNECTED:
            log.log(1,'disconnected from %s:%s' % (self.host, self.specname or self.port))
            SpecEventsDispatcher.emit(self, 'disconnected', ())

    def connect_event(self, signal, cb):
        SpecEventsDispatcher.connect(self, signal, cb)

    def update_events(self):
        SpecEventsDispatcher.dispatch()

    update = update_events

    def disconnect(self):
        """Disconnect from the remote Spec version."""
        self.handle_close()

    def handle_close(self):
        """Handle 'close' event on socket."""

        self.socket_connected = False
        self.server_version = None

        if self.socket:
            self.close()

        self.valid_socket = False

        self.reg_channels = {}
        self.aliasedChannels = {}
        self.spec_disconnected()

    def handle_error(self):
        """Handle an uncaught error."""
        exception, error_string, tb = sys.exc_info()
        # let Python display exception like it wants!
        sys.excepthook(exception, error_string, tb)

    def handle_read(self):
        """Handle 'read' events on socket

        Messages are built from the read calls on the socket.
        """
        _received = self.recv(32768) 

        self.received_strings.append(_received)

        if is_python3():
            s = b''.join(self.received_strings)
            sbuffer = memoryview(s)
        else:
            s = ''.join(self.received_strings)
            sbuffer = buffer(s)

        consumedBytes = 0
        offset = 0

        while offset < len(sbuffer):
            if self.message is None:
                self.message = SpecMessage.message(version = self.server_version)

            consumedBytes = self.message.readFromStream(sbuffer[offset:])

            if consumedBytes == 0:
                break

            offset += consumedBytes

            if self.message.isComplete():
                try:
                    # dispatch incoming message
                    if self.message.cmd == SpecMessage.REPLY:
                        self.dispatch_reply_msg(self.message)
                    elif self.message.cmd == SpecMessage.EVENT:
                        self.dispatch_event_msg(self.message)
                    elif self.message.cmd == SpecMessage.HELLO_REPLY:
                        self.dispatch_hello_reply_msg(self.message)
                except Exception as e:
                    self.message = None
                    self.received_strings = [ s[offset:] ]
                    raise SpecClientProtocolError(str(e))
                else:
                    self.message = None
                                   
        self.received_strings = [ s[offset:] ]

    def dispatch_event_msg(self, msg):

        chan_name = msg.name

        chan = self.reg_channels.get(chan_name, None)

        if chan is None:
            raise SpecClientProtocolError("received event for unregistered channel %s" % chan_name)

        chan.update(msg.data, msg.flags == SpecMessage.DELETED)

    def dispatch_reply_msg(self, msg):
        reply_id = msg.sn

        if reply_id <= 0:
            return 

        reply = self.reg_replies.get(reply_id, None)

        if reply is None:
            raise SpecClientProtocolError("non expected reply received")

        # deliver data to reply
        del self.reg_replies[reply_id]

        is_error = (msg.type == SpecMessage.ERROR)
        errmsg = msg.err
        reply.update(msg.data, is_error, errmsg)

    def dispatch_hello_reply_msg(self, msg):
        if self.check_appname(msg.name):
            self.server_version = msg.vers #header version
            self.spec_connected()
        else:
            self.server_version = None
            self.socket_connected = False
            self.close()
            self.state = DISCONNECTED

    def check_appname(self, name):
        """Check remote Spec version

        If we are in port scanning mode, check if the name from
        Spec corresponds to our required Spec version.
        """
        if self.scanports:
            if name == self.specname:
                return True
            else:
                return False
        else:
            # if connected by port just be happy
            self.specname = name
            return True

    def readable(self):
        return self.valid_socket

    def writable(self):
        """Return True if socket should be written."""
        if not self.readable(): 
            return False

        buf_len = sum(map(len, self.output_strings)) 

        if len(self.sendq) > 0 or buf_len > 0:
            return True
        return False

        #ret = self.readable() and (len(self.sendq) > 0 or sum(map(len, self.output_strings)) > 0)
        #return ret

    def handle_connect(self):
        """Handle 'connect' event on socket

        Send a HELLO message.
        """
        self.socket_connected = True
        self.state = WAITINGFORHELLO
        self.waiting_hello_started = time.time()
        self.send_msg_hello()

    def handle_write(self):
        """Handle 'write' events on socket

        Send all the messages from the queue.
        """
        while len(self.sendq) > 0:
            self.output_strings.append(self.sendq.pop().sendingString())

        if is_python3():
            out_buffer = b''.join(self.output_strings)
        else:
            out_buffer = ''.join(self.output_strings)

        sent = self.send(out_buffer)

        self.output_strings = [ out_buffer[sent:] ]


    def send_msg_cmd_with_return(self, cmd):
        """Send a command message to the remote Spec server, and return the reply id.

        Arguments:
        cmd -- command string, i.e. '1+1'
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        try:
            caller = sys._getframe(1).f_locals['self']
        except KeyError:
            caller = None

        reply, msg = SpecMessage.msg_cmd_with_return(cmd, version = self.server_version)
        reply_id = self.__send_msg_with_reply(reply, msg, receiver_obj = caller)

        return reply_id

    def wait_reply(self, reply_id, timeout=DEFAULT_REPLY_TIMEOUT):

        start_wait = time.time()

        if reply_id not in self.reg_replies:
            return None

        reply = self.reg_replies[reply_id]

        while reply.is_pending():
            elapsed = time.time() - start_wait
            if elapsed > timeout:
                raise SpecClientTimeoutError('timeout waiting for reply')
            time.sleep(0.02)

        return reply.get_data()

    def send_msg_func_with_return(self, cmd):
        """Send a command message to the remote Spec server using the new 'func' feature, and return the reply id.

        Arguments:
        cmd -- command string
        """
        if self.server_version < 3:
            raise SpecClientVersionError("need spec server minimum version 3")
        
        if not self.is_connected():
            raise SpecClientNotConnectedError

        try:
            caller = sys._getframe(1).f_locals['self']
        except KeyError:
            caller = None

        reply, msg = SpecMessage.msg_func_with_return(cmd, version = self.server_version)
        reply_id = self.__send_msg_with_reply(reply, msg, receiver_obj = caller)
        return reply_id

    def send_msg_cmd(self, cmd):
        """Send a command message to the remote Spec server.

        Arguments:
        cmd -- command string, i.e. 'mv psvo 1.2'
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        self.__send_msg_no_reply(SpecMessage.msg_cmd(cmd, version = self.server_version))

    def send_msg_func(self, cmd):
        """Send a command message to the remote Spec server using the new 'func' feature

        Arguments:
        cmd -- command string
        """
        if self.server_version < 3:
            raise SpecClientVersionError("need spec server minimum version 3")

        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_func(cmd, version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_chan_read(self, chan_name):
        """Send a channel read message, and return the reply id.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        try:
            caller = sys._getframe(1).f_locals['self']
        except KeyError:
            caller = None

        reply, msg = SpecMessage.msg_chan_read(chan_name, version = self.server_version)
        reply_id = self.__send_msg_with_reply(reply, msg, receiver_obj = caller)
        return reply_id

    def send_msg_chan_send(self, chan_name, value):
        """Send a channel write message.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        value -- channel value
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_chan_send(chan_name, value, version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_register(self, chan_name):
        """Send a channel register message.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_register(chan_name, version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_unregister(self, chan_name):
        """Send a channel unregister message.

        Arguments:
        chan_name -- a string representing the channel name, i.e. 'var/toto'
        """
        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_unregister(chan_name, version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_close(self):
        """Send a close message."""
        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_close(version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_abort(self):
        """Send an abort message."""
        if not self.is_connected():
            raise SpecClientNotConnectedError

        msg = SpecMessage.msg_abort(version = self.server_version)
        self.__send_msg_no_reply( msg )

    def send_msg_hello(self):
        """Send a hello message."""
        msg = SpecMessage.msg_hello()
        self.__send_msg_no_reply(msg)

    def __send_msg_with_reply(self, reply, msg, receiver_obj = None):
        """Send a message to the remote Spec, and return the reply id.

        The reply object is added to the reg_replies dictionary,
        with its reply id as the key. The reply id permits then to
        register for the reply using the 'registerReply' method.

        Arguments:
        reply -- SpecReply object which will receive the reply
        message -- SpecMessage object defining the message to send
        """
        reply_id = reply.id
        self.reg_replies[reply_id] = reply

        if hasattr(receiver_obj, 'replyArrived'):
            SpecEventsDispatcher.connect(reply, 'replyFromSpec', receiver_obj.replyArrived)

        self.sendq.insert(0, msg)

        return reply_id

    def __send_msg_no_reply(self, msg):
        """Send a message to the remote Spec.

        If a reply is sent depends only on the message, and not on the
        method to send the message. Using this method, any reply is
        lost.
        """
        self.sendq.insert(0, msg)

if __name__ == '__main__':
    import sys
    log.start()

    conn = ThreadedSpecConnection(sys.argv[1])
    conn.wait_connected(timeout=10)
    print("connection is now ready")



#  %W%  %G% CSS
#  "pyspec" Release %R%
#
"""Helper module for managing scans"""

import time
import copy
import tokenize
import cStringIO

import numpy as np

from pyspec.css_logger import log

from SpecConnectionsManager import SpecConnectionsManager
import SpecCommand as SpecCommand
import SpecEventsDispatcher as SpecEventsDispatcher

(TIMESCAN) = (16)

def _iterable(next, terminator):
    out = []
    token = next()
    while token[1] != terminator:
        out.append(_atom(next, token))
        token = next()
        if token[1] == ",":
            token = next()
    return out

def _dictable(next):
    out = []
    token = next()
    while token[1] != '}':
        k = _atom(next, token)
        token = next()
        token = next()
        v = _atom(next, token)
        out.append((k, v))
        token = next()
        if token[1] == ",":
            token = next()
    return dict(out)

def _atom(next, token):
    if token[1] == "(":
        return tuple(_iterable(next, ')'))
    if token[1] == "[":
        return list(_iterable(next, ']'))
    if token[1] == "{":
        return _dictable(next)
    if token[1] == "array":
        token = next()
        return np.array(*_iterable(next, ')'))
    elif token[0] is tokenize.STRING:
        return token[1][1:-1].decode("string-escape")
    elif token[0] is tokenize.NUMBER:
        try:
            return int(token[1], 0)
        except ValueError:
            return float(token[1])
    elif token[1] == "-":
        token = list(next())
        token[1] = "-" + token[1]
        return _atom(next, token)
    elif token[0] is tokenize.NAME:
        if token[1] == 'None':
            return None
        raise ValueError('tokenize NAME: %s unrecognized' % token[1])
    elif not token[0]:
        return
    for i, v in iter(tokenize.__dict__.items()):
        if v == token[0]:
            raise ValueError("tokenize.%s unrecognized: %s" % (i, token[1]))

def simple_eval(source):
    """a safe version of the builtin eval function, """
    src = cStringIO.StringIO(source).readline
    src = tokenize.generate_tokens(src)
    return _atom(src.next, src.next())


class SpecScanA:

    @property
    def paused(self):
        # False when a scan is running or has completed normally
        return self.__status == 'paused'

    @property
    def ready(self):
        # False when a scan starts, only True once a scan completes normally
        return self.__status == 'ready'

    @property
    def scanning(self):
        # True when a scan is running, False when scan completes or is paused
        return self.__status == 'scanning'

    @property
    def specVersion(self):
        return self.__specVersion

    def __init__(self, specVersion = None):
        self.scanParams = {}
        self.scanCounterMne = None
        self.__status = 'ready'
        self.__specVersion = None

        if specVersion is not None:
            self.connectToSpec(specVersion)
        else:
            self.connection = None

    def connectToSpec(self, specVersion):
        self.connection = SpecConnectionsManager().getConnection(specVersion)
        self.__specVersion = specVersion

        SpecEventsDispatcher.connect(self.connection, 'connected',
                                     self.__connected)
        SpecEventsDispatcher.connect(self.connection, 'disconnected',
                                     self.__disconnected)

        if self.connection.isSpecConnected():
            self.__connected()

    def isConnected(self):
        return self.connection and self.connection.isSpecConnected()


    def __connected(self):
       self.connection.registerChannel('status/ready', self.__statusReady,
                                    dispatchMode=SpecEventsDispatcher.FIREEVENT)
       self.connection.registerChannel('var/_SC_NEWSCAN', self.__newScan,
                                    dispatchMode=SpecEventsDispatcher.FIREEVENT)
       self.connection.registerChannel('var/_SC_NEWPLOTDATA',
                                    self.__newScanPoint,
                                    dispatchMode=SpecEventsDispatcher.FIREEVENT)
       self.connection.registerChannel('var/_SC_NEWSCANDATA',
                                    self.__newScanData,
                                    dispatchMode=SpecEventsDispatcher.FIREEVENT)
       self.connected()

    def connected(self):
        pass


    def __disconnected(self):
        self.scanCounterMne = None
        self.__status = 'ready'
        self.scanParams = {}

        self.disconnected()


    def disconnected(self):
        pass


    def getScanType(self):
        try:
            return self.scanParams['scantype']
        except:
            return -1


    def isScanning(self):
        return self.scanning


    def __newScan(self, scanParams):
        if DEBUG: print( "SpecScanA.__newScan", scanParams )

        if not scanParams:
            if self.scanning:
                # receive 0 when scan ends normally
                self.__status = 'ready'
                self.scanFinished()
            return

        if not self.ready:
            # a new scan was started before the old one completed
            # lets provide an opportunity to clean up
            self.scanAborted()

        self.__status = 'scanning'

        self.scanParams = simple_eval(scanParams)

        if not isinstance(self.scanParams, dict):
            return

        self.newScan(self.scanParams)

        self.scanCounterMne = self.scanParams.get('counter')
        if (not self.scanCounterMne) or self.scanCounterMne == '?':
            log.error("No counter selected for scan.")
            self.scanCounterMne = None
            return

        self.scanStarted() # A.B


    def newScan(self, scanParameters):
        pass


    def __newScanData(self, scanData):
        if self.paused and scanData:
            self.__status = 'scanning'
            self.scanResumed()
        if self.scanning and scanData:
            scanData = simple_eval(scanData)

            self.newScanData(scanData)


    def newScanData(self, scanData):
        pass


    def __newScanPoint(self, scanData):
        if self.paused and scanData:
            self.__status = 'scanning'
            self.scanResumed()
        if self.scanning and scanData:
            scanData = simple_eval(scanData)

            i = scanData['i']
            x = scanData['x']
            y = scanData[self.scanCounterMne]

            # hack to know if we should call newScanPoint with
            # scanData or not (for backward compatiblity)
            if len(self.newScanPoint.im_func.func_code.co_varnames) > 4:
              self.newScanPoint(i, x, y, scanData)
            else:
              self.newScanPoint(i, x, y)


    def newScanPoint(self, i, x, y, counters_value):
        pass


    def abort(self):
        if self.isConnected and (self.scanning or self.paused):
            if self.scanning:
                self.connection.abort()
            self.__status = 'ready'
            self.scanAborted()


    def pause(self):
        if self.isConnected() and self.scanning:
            self.connection.abort()


    def resume(self):
        if self.isConnected() and self.paused:
            SpecCommand.SpecCommandA(self.connection, 'scan_on')()

    def scanAborted(self):
        pass

    def scanFinished(self):
        pass

    def scanPaused(self):
        pass

    def scanResumed(self):
        pass

    def scanStarted(self): # A.B
        pass # A.B

    def __statusReady(self, status):
        if status and self.scanning:
            self.__status = 'paused'
            self.scanPaused()

    def ascan(self, motorMne, startPos, endPos, nbPoints, countTime):
        if self.connection.isSpecConnected():
            cmd = "ascan %s %f %f %d %f" % (motorMne, startPos, endPos,
                                            nbPoints, countTime)
            self.connection.send_msg_cmd(cmd)
            return True
        else:
            return False



#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#
# Exception class
#
class SpecClientError(Exception):

    errstr = "pyspec.client error"

    def __init__(self, error = None, err = None):
        Exception.__init__(self)

        self.error = error
        self.err = err

    def __str__(self):
        errstr = self.errstr

        if self.error: 
            errstr += ". " + str(self.error)
        if self.err: 
            errstr += ". " + str(self.err)

        return errstr

class SpecClientProtocolError(SpecClientError):
    errstr = "pyspec.client error (spec protocol problem)"

class SpecClientVersionError(SpecClientError):
    errstr = "pyspec.client error (wrong server version)"

class SpecClientTimeoutError(SpecClientError):
    errstr = "pyspec.client error (request timeout)"

class SpecClientNotConnectedError(SpecClientError):
    errstr = "pyspec.client error (no connection)"



#  %W%  %G% CSS
#  "pyspec" Release %R%
#
import weakref

import time
import inspect
import sys

try:
   import queue
except ImportError:
   import Queue as queue

import saferef 
from pyspec.css_logger import log

DEBUG=4  # debug level for this module

(UPDATEVALUE, FIREEVENT) = (1, 2)

class SpecClientDispatcherError(Exception):
    def __init__(self, args=None):
        self.args = args

def is_python3():
    return sys.version_info[0] == 3

def min_args(slot):
    if is_python3():
        sig = inspect.signature(slot)
        params = sig.parameters.values()
        npars = 0
        for param in params:
            if param.kind == param.POSITIONAL_OR_KEYWORD:
                if param.default == param.empty:
                   npars += 1
        return npars
    else:  # python2
        argspec = inspect.getargspec(slot)
        nargs = len(argspec.args)
        if argspec.defaults:
            nargs -= len(argspec.defaults)
        return nargs

def robustApply(slot, arguments = ()):
    """Call slot with appropriate number of arguments"""
    if inspect.isclass(slot):
        slot = slot.__call__

    if hasattr(slot, 'im_func'):
        # an instance method
        if is_python3():
            margs = min_args(slot.__func__) - 1
        else:
            margs = min_args(slot.im_func) - 1
    else:
        margs = min_args(slot) 

    if len(arguments) < margs:
        msg = 'Not enough arguments for calling slot %s ' \
              '(need: %d, given: %d)' % (repr(slot), margs, len(arguments))
        raise SpecClientDispatcherError(msg)
    else:
        return slot(*arguments[0:margs])


class Receiver:
    def __init__(self, weakReceiver, dispatchMode):
        self.weakReceiver = weakReceiver
        self.dispatchMode = dispatchMode


    def __call__(self, arguments):
        slot = self.weakReceiver() # get the strong reference

        if slot is not None:
            return robustApply(slot, arguments)


class Event:
    def __init__(self, sender, signal, arguments):
        self.receivers = []
        senderId = id(sender)
        signal = str(signal)
        self.args = arguments

        try:
            self.receivers = connections[senderId][signal]
        except:
            pass


class EventQueue(object):
    queue = None

    def __init__(self):
        if EventQueue.queue is None:
            EventQueue.queue = queue.Queue(0)

    def empty(self):
        return EventQueue.queue.empty()

    def get(self):
        return EventQueue.queue.get(False)

    def put(self, event):
        """Put an event into the queue."""
        receiversList = event.receivers

        EventQueue.queue.mutex.acquire()

        try:
            showstatus()
            was_empty = not EventQueue.queue._qsize()

            for r in receiversList:
                if not was_empty:
                    if r.dispatchMode == UPDATEVALUE:
                        for i in range(len(EventQueue.queue.queue)):
                            _r, args = EventQueue.queue.queue[i]
                            if r == _r:
                                del EventQueue.queue.queue[i]
                                break

                EventQueue.queue._put( (r, event.args) )
        except:
            import traceback
            log.log(DEBUG,"could not add event to queue %s" % traceback.format_exc())
        finally:
            EventQueue.queue.mutex.release()

eventsToDispatch = EventQueue()
connections = {} # { senderId0: { signal0: [receiver0, ..., receiverN], signal1: [...], ... }, senderId1: ... }
senders = {} # { senderId: sender, ... }

def callableObjectRef(object):
    """Return a safe weak reference to a callable object"""
    return saferef.safe_ref(object, _removeReceiver)

def connect(sender, signal, slot, dispatchMode = UPDATEVALUE):
    if sender is None or signal is None:
        return

    if not callable(slot):
        return

    senderId = id(sender)
    signal = str(signal)
    signals = {}

    if senderId in connections:
        signals = connections[senderId]
    else:
        connections[senderId] = signals

    def remove(object, senderId=senderId):
        _removeSender(senderId)

    try:
        weakSender = weakref.ref(sender, remove)
        senders[senderId] = weakSender
    except:
        pass

    receivers = []

    if signal in signals:
        receivers = signals[signal]
    else:
        signals[signal] = receivers

    weakReceiver = callableObjectRef(slot)

    for r in receivers:
        if r.weakReceiver == weakReceiver:
            r.dispatchMode = dispatchMode
            return

    receivers.append(Receiver(weakReceiver, dispatchMode))

def disconnect(sender, signal, slot):
    if sender is None or signal is None:
        return

    if not callable(slot):
        return

    senderId = id(sender)
    signal = str(signal)

    try:
        signals = connections[senderId]
    except KeyError:
        return
    else:
        try:
            receivers = signals[signal]
        except KeyError:
            return
        else:
            weakReceiver = callableObjectRef(slot)

            toDel = None
            for r in receivers:
                if r.weakReceiver == weakReceiver:
                    toDel = r
                    break

            if toDel is not None:
                receivers.remove(toDel)

                # log.log(DEBUG, "cleaning up connections, because sender is removed %s" % senderId)
                _cleanupConnections(senderId, signal)
    
def showstatus():

    return
    log.log(DEBUG,"status of connections is:")

    for i in connections.keys():
        try:
           name = senders[i]().name
           if name.startswith('motor'):
               continue
           log.log(DEBUG,"(1)  - %s " % senders[i]().name)
        except:
           log.log(DEBUG,"(2)  - %s " % str(senders[i]()))

def emit(sender, signal, arguments = ()):
    try:
        ev = Event(sender, signal, arguments)
        eventsToDispatch.put(ev)
    except:
        log.log(DEBUG,"failed adding event")
        import traceback
        log.log(DEBUG, traceback.format_exc())
    #if threading.current_thread() == MAIN_THREAD:
    #    dispatch(-1)


def dispatch(max_time_in_s=1):
    t0 = time.time()
    while True:
        try:
            if eventsToDispatch.empty():
                break
            receiver, args = eventsToDispatch.get()
        except queue.Empty:
            log.log(2, "uhmmm")
            break
        except:
            log.log(1, "other exception while dispatching events")
            import traceback
            log.log(1, traceback.format_exc())
        else:
            receiver(args)
            if max_time_in_s < 0:
              continue
            elif (time.time()-t0) >= max_time_in_s:
              break

def _removeSender(senderId):
    try:
        del connections[senderId]
        del senders[senderId]
    except KeyError:
         pass

def _removeReceiver(weakReceiver):
    """Remove receiver from connections"""
    #log.log(DEBUG, "cleaning up connections, because receiver is removed %s" % str(weakReceiver))
    return
    for senderId in list(connections.keys()):
        for signal in list(connections[senderId].keys()):
            receivers = connections[senderId][signal]

            for r in receivers:
                    log.log(DEBUG, "cleaning up connections, because receiver is removed %s" % str(r))
                    receivers.remove(r)
                    break

            log.log(DEBUG, "cleaning up connections, because receiver is removed for signal %s" % str(signal))
            _cleanupConnections(senderId, signal)


def _cleanupConnections(senderId, signal):
    """Delete any empty signals for sender. Delete sender if empty."""

    receivers = connections[senderId][signal]

    if len(receivers) == 0:
        # no more receivers
        signals = connections[senderId]
        del signals[signal]

        if len(signals) == 0:
            # no more signals
            _removeSender(senderId)





#  %W%  %G% CSS
#  "pyspec" Release %R%
#
"""SpecMessage module

This module defines classes and functions for creating messages
from data received from Spec, and for generating messages to be
sent to Spec.

It handles the different message versions (headers 2, 3 and 4).

For message description check the protocol section of the spec
"help server" page or 
  http://certif.com/spec_help/server.html

Message versions:
   V2 - spec 5.04.03 - released July 20, 2003
   V3 - spec 5.04.04 - released July 18, 2004
   V4 - spec 5.04.05 and later - released Sept 30, 2004
"""

import struct
import time
import types

from pyspec.utils import is_python3, is_python2
from pyspec.css_logger import log

from SpecReply import SpecReply
import SpecArray 

DEBUG=4  # debug level for this module

(DOUBLE, STRING, ERROR, ASSOC) = (1,2,3,4)

MAGIC_NUMBER=4277009102
NATIVE_HEADER_VERSION=4
NULL_B=b'\000'
NULL='\000'

# cmd
(CLOSE, ABORT, CMD, CMD_WITH_RETURN, REGISTER, UNREGISTER, EVENT,  \
 FUNC, FUNC_WITH_RETURN, CHAN_READ, CHAN_SEND, REPLY, HELLO, HELLO_REPLY) = \
                                         (1,2,3,4,6,7,8,9,10,11,12,13,14,15)

# flags
DELETED = 0x0001

def message(*args, **kwargs):
    """Return a new SpecMessage object

    The returned SpecMessage object can be of any of the available message
    class. You can specify the desired message class with the 'version' keyword
    argument. If not specified, defaults to NATIVE_HEADER_VERSION.

    Arguments are passed to the appropriate message constructor.

    Keyword arguments:
    version -- message version, defaults to NATIVE_HEADER_VERSION. When
    reading messages from stream, you can set it to None and it will try
    to guess the suitable message class by reading the header version
    from Spec.
    """
    version = kwargs.get('version', NATIVE_HEADER_VERSION)
    order = kwargs.get('order', '<')

    if len(order) == 0:
      order = "<"

    if version == 4:
        m = message4(*args)
    elif version == 3:
        m = message3(*args)
    elif version == 2:
        m = message2(*args)
    else:
        m = anymessage(*args) # only for reading message from stream

    m.packedHeaderDataFormat=order+m.packedHeaderDataFormat[1:]

    return m


def rawtodictonary(rawstring):
    """Transform a list as coming from a SPEC associative array
    to a dictonary - 2dim arrays are transformed top dict with dict
    entries. In SPEC the key contains \x1c"""

    if is_python3():
        rawstring = bytes(rawstring).decode('utf-8')  
    
    raw = rawstring.split(NULL)[:-2]

    data = {}
    for i in range(0,len(raw) - 1,2):

        key,val = raw[i], raw[i+1]
        keyel = key.split("\x1c")

        if len(keyel) == 1:
            if key in data:
              data[key][None] = val
            else:
              data[key]=val
        else:
            if keyel[0] in data and not isinstance(data[keyel[0]], dict):
              data[keyel[0]]={ None: data[keyel[0]] }

            try:
                data[keyel[0]][keyel[1]] = val
            except TypeError:
                data[keyel[0]] = {keyel[1] : val}
            except KeyError:
                data[keyel[0]] = {keyel[1] : val}

    return data

def dictionarytoraw(dict):
    """Transform a Python dictionary object to the string format
    expected by Spec"""

    data = ""

    for key, val in dict.items():
        if isinstance(val,dict):
            for kkey, vval in iter(val.items()):
                if kkey is None:
                  data += str(key) + NULL + str(vval) + NULL
                else:
                  data += ''.join([str(key), '\x1c', str(kkey), NULL, str(vval), NULL])
        else:
            data += str(key) + NULL + str(val) + NULL

    return (len(data) > 0 and data) or NULL

class SpecMessage:
    """Base class for messages."""
    def __init__(self, packedHeader):
        """Constructor

        Arguments:
        packedHeader -- string representing the packed header format for the message,
        use the same syntax as the 'struct' Python module
        """
        self.packedHeaderDataFormat = packedHeader
        self.headerLength = struct.calcsize(self.packedHeaderDataFormat)
        self.bytesToRead = self.headerLength
        self.readheader = True
        self.data = ''
        self.type = None

        #message properties
        self.magic = None
        self.vers = None
        self.size = None
        self.sn = None
        self.sec = None
        self.usec = None
        self.cmd = None
        self.type = None
        self.rows = 0
        self.cols = 0
        self.name = None
        self.err = 0
        self.flags = 0

    def isComplete(self):
        """Return wether a message read from stream has been fully received or not."""
        return self.bytesToRead == 0

    def readFromStream(self, streamBuf):
        """Read buffer from stream and try to create a message from it

        Arguments:
        streamBuf - string buffer of the last bytes received from Spec

        Return value :
        the number of consumed bytes
        """
        consumedBytes = 0

        try:
            while self.bytesToRead > 0 and len(streamBuf[consumedBytes:]) >= self.bytesToRead:
                if self.readheader:
                    self.readheader = False
                    self.type, self.bytesToRead = self.readHeader(streamBuf[:self.headerLength])
                    consumedBytes = self.headerLength
                else:
                    rawdata = streamBuf[consumedBytes:consumedBytes+self.bytesToRead]
                    consumedBytes += self.bytesToRead
                    self.bytesToRead = 0
    
                    self.data = self.readData(rawdata, self.type)
    
        except BaseException as e:
            import traceback
            log.log(2, "error reading message from stream %s" % str(e))
            log.log(2,  traceback.format_exc())
  
        return consumedBytes

    def readHeader(self, rawstring):
        """Read the header of the message coming from stream

        Arguments:
        rawstring -- raw bytes of the header

        Return value:
        (message data type, message data len) tuple
        """
        return (None, 0)

    def readData(self, rawstring, datatype):
        """Read the data part of the message coming from stream

        Arguments:
        rawstring -- raw data bytes
        datatype -- data type

        Return value:
        the data read
        """
        data = rawstring[:-1] #remove last NULL byte

        if datatype == ERROR:
            if is_python3():
                data2 = data.tobytes()
                data = data2.decode('utf-8')
            return data
        elif datatype == STRING or datatype == DOUBLE:
            # try to convert data to a more appropriate type
            if is_python3():
                data2 = data.tobytes()
                data = data2.decode('utf-8')

            try:
                data = int(data)
            except:
                try:
                    data = float(data)
                except:
                    pass

            return data
        elif datatype == ASSOC:
            return rawtodictonary(rawstring)
        elif SpecArray.isArrayType(datatype):
            #Here we read cols and rows... which are *supposed* to be received in the header!!!
            #better approach: data contains this information (since it is particular to that data type)
            return SpecArray.SpecArray(rawstring, datatype, self.rows, self.cols)
        else:
            raise TypeError


    def dataType(self, data):
        """Try to guess data type

        Works for obvious cases only
          - it is a hard job guessing ARRAY_* types, we ignore this case (user has to provide a suitable datatype)
          - we cannot make a difference between ERROR type and STRING type
        """
        if isinstance(data,str):
            return STRING
        elif isinstance(data,dict):
            return ASSOC
        elif isinstance(data,int) or isinstance(data,float):
            return STRING
        elif is_python2() and isinstance(data,long):
            return STRING
        elif isinstance(data, SpecArray.SpecArrayData):
            self.rows, self.cols = data.shape
            return data.type

    def sendingDataString(self, data, datatype):
        """Return the string representing the data part of the message."""
        rawstring = ''

        if datatype in (ERROR, STRING, DOUBLE):
            rawstring = str(data)
        elif datatype == ASSOC:
            rawstring = dictionarytoraw(data)
        elif SpecArray.isArrayType(datatype):
            rawstring = data.tostring()

        if len(rawstring) > 0:
            rawstring += NULL

        return rawstring

    def sendingString(self):
        """Create a string representing the message which can be send
        over the socket."""
        return ''

class message2(SpecMessage):
    """Version 2 message class"""
    def __init__(self, *args, **kwargs):
        """Constructor

        If called without arguments, message is supposed to be read from stream.
        Otherwise, the 'init' method is called with the specified arguments, for
        creating a message from arguments.
        """
        SpecMessage.__init__(self, '<IiiiIIiiIII80s')

        if len(args) > 0:
            self.init(*args, **kwargs)

    def init(self, ser, cmd, name, data, datatype = None, rows = 0, cols = 0):
        """ Create a message from the arguments"""
        self.vers = 2 #header version
        self.size = self.headerLength
        self.magic = MAGIC_NUMBER
        self.rows = rows
        self.cols = cols
        self.data = data
        self.type = datatype or self.dataType(self.data)
        self.time = time.time()
        self.sec = int(self.time)
        self.usec = int((self.time-self.sec)*1E6)
        self.sn, self.cmd, self.name = ser, cmd, str(name)

    def readHeader(self, rawstring):
        self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)
        if self.magic != MAGIC_NUMBER:
            self.packedHeaderDataFormat=">"+self.packedHeaderDataFormat[1:]
            self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)

        self.time = self.sec + float(self.usec) / 1E6

        if is_python3():
            self.name = name.replace(NULL_B, b'') #remove padding null bytes
        else:
            self.name = name.replace(NULL, '') #remove padding null bytes

        return (datatype, datalen)

    def sendingString(self):
        if self.type is None:
            # invalid message
            return ''

        data = self.sendingDataString(self.data, self.type)
        datalen = len(data)

        if is_python3():
            name = self.name.encode('utf-8')
            data = data.encode('utf-8')
        else:
            name = str(self.name)

        header = struct.pack(self.packedHeaderDataFormat, self.magic, self.vers, self.size,
                             self.sn, self.sec, self.usec, self.cmd, self.type,
                             self.rows, self.cols, datalen, name)
        return header + data

class message3(SpecMessage):
    def __init__(self, *args, **kwargs):
        SpecMessage.__init__(self, '<IiiiIIiiIIIi80s')

        if len(args) > 0:
            self.init(*args, **kwargs)

    def init(self, ser, cmd, name, data, datatype = None, rows = 0, cols = 0):
        """ Create a message from the arguments """
        self.vers = 3 #header version
        self.size = self.headerLength
        self.magic = MAGIC_NUMBER
        self.rows = rows
        self.cols = cols
        self.data = data
        self.type = datatype or self.dataType(self.data)
        self.time = time.time()
        self.sec = int(self.time)
        self.usec = int((self.time-self.sec)*1E6)
        self.sn, self.cmd, self.name = ser, cmd, str(name)

    def readHeader(self, rawstring):
        self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, self.err, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)

        if self.magic != MAGIC_NUMBER:
            self.packedHeaderDataFormat=">"+self.packedHeaderDataFormat[1:]
            self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, self.err, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)

        self.time = self.sec + float(self.usec) / 1E6

        if is_python3():
            self.name = name.replace(NULL_B, b'') # remove padding null bytes
        else:
            self.name = name.replace(NULL, '') # remove padding null bytes

        if self.err > 0:
            datatype = ERROR #change message type to 'ERROR' for further processing

        return (datatype, datalen)

    def sendingString(self):
        if self.type is None:
            return ''

        data = self.sendingDataString(self.data, self.type)
        datalen = len(data)

        if is_python3():
            name = self.name.encode('utf-8')
            data = data.encode('utf-8')
        else:
            name = str(self.name)

        header = struct.pack(self.packedHeaderDataFormat, self.magic, self.vers, self.size,
                             self.sn, self.sec, self.usec, self.cmd, self.type,
                             self.rows, self.cols, datalen, self.err, self.name)

        return header + data

class message4(SpecMessage):
    def __init__(self, *args, **kwargs):
        SpecMessage.__init__(self, '<IiIIIIiiIIIii80s')

        if len(args) > 0:
            self.init(*args, **kwargs)

    def init(self, ser, cmd, name, data, datatype = None, rows = 0, cols = 0):
        """ Create a message from the arguments """
        self.vers = 4 #header version
        self.size = self.headerLength
        self.magic = MAGIC_NUMBER
        self.rows = rows
        self.cols = cols
        self.data = data
        self.type = datatype or self.dataType(self.data)
        self.time = time.time()
        self.sec = int(self.time)
        self.usec = int((self.time-self.sec)*1E6)
        self.sn, self.cmd, self.name = ser, cmd, str(name)


    def readHeader(self, rawstring):
        self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, self.err, self.flags, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)
        if self.magic != MAGIC_NUMBER:
            self.packedHeaderDataFormat=">"+self.packedHeaderDataFormat[1:]
            self.magic, self.vers, self.size, self.sn, \
                    self.sec, self.usec, self.cmd, \
                    datatype, self.rows, self.cols, \
                    datalen, self.err, self.flags, name  = struct.unpack(self.packedHeaderDataFormat, rawstring)
        self.time = self.sec + float(self.usec) / 1E6
        
        if is_python3():
            name = name.decode()
        
        name = name.replace(NULL, '') #remove padding null bytes
        self.name = name.strip()

        if self.err > 0:
            datatype = ERROR #change message type to 'ERROR' for further processing

        return (datatype, datalen)

    def sendingString(self):
        if self.type is None:
            # invalid message
            return ''

        try:
            data = self.sendingDataString(self.data, self.type)
            datalen = len(data)

            if is_python3():
                name = self.name.encode('utf-8')
                data = data.encode('utf-8')
            else:
                name = str(self.name)

            header = struct.pack(self.packedHeaderDataFormat, self.magic, self.vers, self.size,
                             self.sn, self.sec, self.usec, self.cmd, self.type,
                             self.rows, self.cols, datalen, self.err, self.flags, name)
        except:
            import traceback
            log.log(2,"specmessage error: %s" % traceback.format_exc())

        return header + data

class anymessage(SpecMessage):
    def __init__(self, *args, **kwargs):
        SpecMessage.__init__(self, '<Ii')

    def readFromStream(self, streamBuf):
        if len(streamBuf) >= self.bytesToRead:
            magic, version = struct.unpack(self.packedHeaderDataFormat, streamBuf[:self.headerLength])

            if magic != MAGIC_NUMBER:
                self.packedHeaderDataFormat=">"+self.packedHeaderDataFormat[1:]
                magic, version = struct.unpack(self.packedHeaderDataFormat, streamBuf[:self.headerLength])

            # try to guess which message class suits best
            if version == 2:
                self.__class__ = message2
                message2.__init__(self)
                return self.readFromStream(streamBuf)
            elif version == 3:
                self.__class__ = message3
                message3.__init__(self)
                return self.readFromStream(streamBuf)
            elif version >= 4:
                self.__class__ = message4
                message4.__init__(self)
                return self.readFromStream(streamBuf)

        return 0

def commandListToCommandString(cmdlist):
    """Convert a command list to a Spec command string."""

    if isinstance(cmdlist, list) and len(cmdlist) > 0:
        cmd = [str(cmdlist[0])]

        for arg in cmdlist[1:]:
            argstr = repr(arg)

            if isinstance(arg,dict):
                argstr = argstr.replace('{', '[')
                argstr = argstr.replace('}', ']')

            cmd.append(argstr)

        return NULL.join(cmd)
    else:
        return ''

def msg_cmd_with_return(cmd, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a command with return message"""
    return message_with_reply(CMD_WITH_RETURN, "", cmd, version, order)

def msg_func_with_return(cmd, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a func with return message"""
    cmd = commandListToCommandString(cmd)
    return message_with_reply(FUNC_WITH_RETURN, "", cmd, version, order)

def msg_cmd(cmd, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a command without reply message"""
    return message_no_reply(CMD, "", cmd, version, order)

def msg_func(cmd, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a func without reply message"""
    cmd = commandListToCommandString(cmd)
    return message_no_reply(FUNC, "", cmd, version, order)

def msg_chan_read(channel, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a property-reading message"""
    return message_with_reply(CHAN_READ, channel, "", version, order)

def msg_chan_send(channel, value, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a property-setting message"""
    return message_no_reply(CHAN_SEND, channel, value, version, order)

def msg_event(channel, value, version = NATIVE_HEADER_VERSION, order="<"):
    """Return an event message"""
    return message_no_reply(EVENT, channel, value, version, order)

def msg_register(channel, version = NATIVE_HEADER_VERSION, order="<"):
    """Return a register message"""
    return message_no_reply(REGISTER, channel, "", version, order)

def msg_unregister(channel, version = NATIVE_HEADER_VERSION, order="<"):
    """Return an unregister message"""
    return message_no_reply(UNREGISTER, channel, "", version, order)

def msg_close(version = NATIVE_HEADER_VERSION, order="<"):
    """Return a close message"""
    return message_no_reply(CLOSE, "", "", version, order)

def msg_abort(version = NATIVE_HEADER_VERSION, order="<"):
    """Return an abort message"""
    return message_no_reply(ABORT, "", "", version, order)

def msg_hello(version = NATIVE_HEADER_VERSION, order="<"):
    """Return a hello message"""
    return message_no_reply(HELLO, "python", "", version, order)

def msg_hello_reply(reply_id, serverName, version = NATIVE_HEADER_VERSION, order="<"):
    return message(reply_id, HELLO_REPLY, serverName, serverName, version = version, order=order)

# Methods to send any Messages
def message_with_reply(cmd, name, data, version = NATIVE_HEADER_VERSION, order="<"):
    """ Lower level call to send a message of a certain type """

    new_reply = SpecReply()
    reply_id = new_reply.id

    msg = message(reply_id, cmd, name, data, version = version, order=order)

    return (new_reply, msg)

def message_no_reply(cmd, name, data, version = NATIVE_HEADER_VERSION, order="<"):
    """ Send a message which will not result in a reply from the server.
    If a reply is sent depends only on the cmd and not on the method
    to send the message """
    return message(0, cmd, name, data, version = version, order=order)

def reply_message(reply_id, name, data, version = NATIVE_HEADER_VERSION, order="<"):
    return message(reply_id, REPLY, name, data, version = version, order=order)

def error_message(reply_id, name, data, version = NATIVE_HEADER_VERSION, order="<"):
    return message(reply_id, REPLY, name, data, ERROR, version = version, order=order)




from pyspec.css_logger import log
from pyspec.client.SpecConnection import SpecConnection
import pyspec.client

def test_splot():
    ldt = pyspec.client.variable(spec, 'LDT')
    print(ldt.get())
    
def test_channels():
    chan = spec.get_channel('var/MYVAR')
    print(chan.read())
    chan.write(45)
    print(chan.read())

def test_motors():
    mot = spec.get_motor('chi')
    print("Position: %s" % mot.position)
    print("Velocity: %s" % mot.read("slew_rate"))
    mot.mv(1)

def test_variables():
    pass

def test_commands():
    pass

spec = SpecConnection('fourc')

test_splot()

#test_channels()
#test_motors()




#  %W%  %G% CSS
#  "pyspec" Release %R%
#

import re
import time
import socket
import asyncore

from pyspec.utils import is_python3
from pyspec.css_logger import log

from SpecConnection import MIN_PORT, MAX_PORT
import SpecMessage as SpecMessage

import spec_updater 

class SpecHandler(asyncore.dispatcher):

    def __init__(self, request, client_address, server):
        asyncore.dispatcher.__init__(self, request)

        # not used
        self.client_address = client_address

        self.server = server
        self.sendq = []

        self.received_strings = []
        self.output_strings = []
        self.client_version = None
        self.client_order = ""

        self.message = None

    # asyncore interface
    def writable(self):
        return len(self.sendq) > 0 or sum(map(len, self.output_strings)) > 0

    def handle_read(self):
        try:
            received = self.recv(32768)

            self.received_strings.append(received)

            if is_python3():
                s = b''.join(self.received_strings)
                sbuffer = memoryview(s)
            else:
                s = ''.join(self.received_strings)
                sbuffer = buffer(s)

            consumedBytes = 0
            offset = 0
            received_messages = []

            while offset < len(sbuffer):

                if self.message is None:
                    try:
                        self.message = SpecMessage.message(version = self.client_version, order=self.client_order)
                    except:
                        import traceback
                        log.log(2, traceback.format_exc())

                consumedBytes = self.message.readFromStream(sbuffer[offset:])
    
                if consumedBytes == 0:
                    break

                offset += consumedBytes

                if self.message.isComplete():
                    # dispatch incoming message
                    if self.message.cmd == SpecMessage.HELLO:
                        self.client_order = self.message.packedHeaderDataFormat[0]
                        self.client_version = self.message.vers
                        self.clientName = self.message.name
                        self.send_hello_reply(self.message.sn, str(self.server.name))
                    else:
                        received_messages.append(self.message)

                    self.message = None

            self.received_strings = [ s[offset:] ]

            for message in received_messages:
                if not self.dispatch_message(message):
                   self.send_error(message.sn, '', 'unsupported command type : %d' % message.cmd)
        except:
            import traceback
            log.log(3,"SpecServer read error. %s" % traceback.format_exc())
            return

    def handle_write(self):
        #
        # send all the messages from the queue
        #
        while len(self.sendq) > 0:
            self.output_strings.append(self.sendq.pop().sendingString())

        try:
            outbuf = b''.join(self.output_strings)

            sent = self.send(outbuf)
            self.output_strings = [ outbuf[sent:] ]
        except:
            import traceback
            log.log(3,"error writing message: %s", traceback.format_exc())

    def handle_close(self):
        self.close()
        try:
            self.server.clients.remove(self)
        except:
            log.log(2,"removing client from spec server. but it is gone already")
            pass

    # END asyncore interface 

    def dispatch_message(self, message):
        try:
            if message.cmd == SpecMessage.CHAN_READ:
                # temporary code (workaround for a Spec client bug)
                self.get_and_reply(reply_id=message.sn, channame=message.name)
            elif message.cmd == SpecMessage.CHAN_SEND:
                self.set_and_reply(reply_id=message.sn, channame=message.name, 
                        value=message.data)
            elif message.cmd in (SpecMessage.CMD_WITH_RETURN, SpecMessage.FUNC_WITH_RETURN):
                self.run_and_reply(reply_id=message.sn, cmd=message.data)
            elif message.cmd == SpecMessage.FUNC_WITH_RETURN:
                self.run_and_reply(reply_id=message.sn, cmd=message.data)
            elif message.cmd == SpecMessage.CMD:
                # in this case we allow for multiple commands separated by colon
                cmdstr = message.data
                cmds = cmdstr.split(";")
                for cmd in cmds:
                    self.run_and_reply(reply_id=message.sn, cmd=cmd)
            elif message.cmd == SpecMessage.REGISTER:
                if message.name == 'update':
                    log.log(3,"update channel registered !")
                    self.updateRegistered = True
            else:
                return False
            return True
        except:
            import traceback
            log.log(2,traceback.format_exc())
            return False
    
    def get_and_reply(self, reply_id, channame):

        ret = self.server.get_value(channame)

        if ret is None:
            self.send_error(reply_id, '', 'cannot get channel ' + channame)
        else:
            self.send_reply(reply_id, '', ret)

    def set_and_reply(self, reply_id, channame, value):

        try:
            ret = self.server.set_value(channame, value)
        except Exception:
            import traceback
            log.log(2, traceback.format_exc())

        if ret is None:
            self.send_error(reply_id, '', 'cannot set channel ' + channame)
        else:
            self.send_reply(reply_id, '', ret)

    def run_and_reply(self, reply_id = None, cmd = '', *args):

        if len(cmd) == 0 or reply_id is None:
            return

        if len(args) == 0:
            cmdstr = str(cmd)
            command, args = self.parse_command(cmdstr)
        else:
            command = cmd

        func = None

        if command in self.server.commands:
            func = self.server.commands[ command ][0]
        elif hasattr(self, command):
            func = getattr(self, command)
        elif hasattr(self.server, command):
            func = getattr(self.server, command)
        else:
            self.send_error(reply_id, '', '"' + command + '" command does not exist.')
            return

        if callable(func):
            try:
                ret = func(*args)
            except:
                import traceback
                traceback.print_exc()

                self.send_error(reply_id, '', 'Failed to execute command "' + command + '"')
            else:
                if ret is None:
                    self.send_error(reply_id, '', command + ' returned None.')
                else:
                    self.send_reply(reply_id, '', str(ret))
        else:
            self.send_error(reply_id, '',  command + ' is not callable on server.')

    def send_hello_reply(self, reply_id, serverName):
        self.sendq.append(SpecMessage.msg_hello_reply(reply_id, serverName, \
                version = self.client_version, order=self.client_order))

    def send_reply(self, reply_id, name, data):
        self.sendq.append(SpecMessage.reply_message(reply_id, name, data, \
                version = self.client_version, order=self.client_order))

    def send_error(self, reply_id, name, data):
        self.sendq.append(SpecMessage.error_message(reply_id, name, data, \
                version = self.client_version, order=self.client_order))

    def send_msg_event(self, chanName, value, broadcast=True):
        self.sendq.append(SpecMessage.msg_event(chanName, value, \
                version = self.client_version, order=self.client_order))
        if broadcast:
            for client in self.server.clients:
                client.send_msg_event(chanName, value, broadcast=False)

    def parse_command(self, cmdstr):

        if SpecMessage.NULL in cmdstr:
            cmdparts = cmdstr.split(SpecMessage.NULL)
            command = cmdparts[0]
            args = tuple([ eval(cmdpart) for cmdpart in cmdparts[1:] ])
            return command, args

        cmd_with_parenthesis = (cmdstr.find('(') >= 0) and True or False

        if cmd_with_parenthesis:
            parent_starts = cmdstr.find('(')
            try:
                command = cmdstr[:cmdpartLength]
                args = eval(cmdstr[cmdpartLength:])
            except:
                print( 'error parsing command string %s' % cmdstr )
                return '', ()
            else:
                if not isinstance(args,tuple):
                    args = (args, )
                return command, args

        # no parenthesis (no else necessary)
        if not cmd_with_parenthesis:
            parts = re.split("\s+", cmdstr.strip())
            if len(parts) > 1:
               command = parts[0]
               args    = parts[1:]
               return command, args
            else:
               return cmdstr, ()

class SpecServer(asyncore.dispatcher):

    def __init__(self, host=None, name=None, allow_name_change=True, 
            handler=SpecHandler):

        asyncore.dispatcher.__init__(self)

        self.handler_class = handler
        self.updater = None
        self.last_print = time.time()

        if host is None:
            self.host = "localhost"
        else:
            self.host = host  # only needed to select a particular ip in the computer

        #
        self.last_log_print = time.time()
        self.log_period = 10

        self.name = None
        self.allow_name_change = allow_name_change

        self.clients = []

        self.commands = {
                "?":  [self.get_help,],
                "command_list":  [self.get_command_list,],
                "channel_list":  [self.get_channel_list,],
                }

        self.channels = {}

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()

        if name is not None:
            self.set_name(name)
        else:
            # delay socket creation
            log.log(2, "SpecServer created without a name. set a name to start it")

    def set_commands(self, cmds):
        self.commands.update(cmds)

    def set_command(self, cmdname, cmdinfo):
        if isinstance(cmdinfo , list):
            self.commands[cmdname] = cmdinfo
        else:
            self.commands[cmdname] = (cmdinfo,)

    def set_channels(self, channels):
        self.channels.update(channels)

    def set_channel(self, channel_name, channel_info):
        self.channels[channel_name] = channel_info

    def get_value(self, name):
        if name in self.channels:
            getcmd = self.channels[name][1]
            if getcmd:
                return getcmd()
            else:
                return None
        return None

    def set_value(self, name, value):
        if name in self.channels:
            setcmd = self.channels[name][0]
            if setcmd:
                return setcmd(value)
            else:
                return none
        return None

    def get_help(self):
        help_str = """This is a demo spec server
Public channels are:
%s
Public commands are:
%s
        """ %  (self.get_channel_list(), self.get_command_list())
        return help_str

    def get_command_list(self):
        return ",".join(self.commands.keys())

    def get_channel_list(self):
        return ",".join(self.channels.keys())

    # sets a name and start server (open socket) if not started yet
    def set_name(self, name):

        if self.name is not None and name != self.name:
            new_name = True
        else:
            new_name = False

        if not self.allow_name_change and new_name:
            if self.name is not None and name != self.name:
                return

        if name is None:
            log.log(2, "cannot start SpecServer without a name.") 
            return

        self.name = name

        if new_name:
            log.log(2, "server name has been changed.")
            return 

        # start only once and when a name is provided
        # a name change does not change the port

        if isinstance(self.name, str): 
            # choose a port number from PORT range
            host = ""
            for p in range(MIN_PORT, MAX_PORT):
                self.server_address = (host,p)

                try:
                    self.bind(self.server_address)
                    self.bind_ok = True
                    break
                except:
                    # already used. continue
                    continue
            else:
                log.log(2, "ALL server addresses are taken. Cannot start")
        else:
            # it is a port number. use that one
            self.server_address = (self.host, self.name)
            try:
                self.bind(self.server_address)
                self.bind_ok = True
            except:
                log.log(2, "Cannot start server on port %s. Already used?" % self.name)

        if self.bind_ok:
            log.log(2,"spec server listening on %s" % str(self.server_address))
            self.listen(5)

    def handle_accept(self):
        try:
            conn, addr = self.accept()
        except:
            return
        else:
            conn.setblocking(0)
            self.clients.append(self.handler_class(conn, addr, self))

    def get_info(self):
        return {'running': self.is_running(), 
                'allow_name_change': self.allow_name_change,
                'name': self.name}

    # auto 
    def run(self):

        if self.is_running():
           return

        # start a thread for automatic update
        self.updater = spec_updater.spec_updater(method=spec_updater.THREAD, 
                update_func=self._update)

        log.log(2, "starting spec server with name: %s" % self.name)
        self.updater.start() 

    def _update(self):

        asyncore.loop(timeout=1,count=1)

        if time.time() - self.last_log_print > self.log_period:
            log.log(4, "command server is running")
            self.last_log_print = time.time()

    def is_running(self):
        if self.updater is None:
            return False
        return self.updater.is_running()

    def stop(self):
        self.updater.stop() 

    def serve_forever(self):
        asyncore.loop()

if __name__ == "__main__":
    import sys

    global myvar
    myvar = 5

    def say_hello(name):
        print("hello "+name)
        return "done"

    def set_var(value):
        global myvar
        log.log(2, "setting variable to %s" % value)
        myvar = value
        return myvar

    def get_var():
        return myvar
        return 5

    def sum_data(data):
        print("received data %s" % str(data))

    log.start()
    name  = sys.argv[1]
    server = SpecServer(name=name)
    server.set_commands({"hello": [say_hello,]})
    server.set_channel("myvar", [set_var, get_var])
    server.run()

    while True:
        try:
            time.sleep(0.2)
        except KeyboardInterrupt:
            server.stop()
            break




#******************************************************************************
#
#  %W%  %G% CSS
#
#  "splot" Release %R%
#
#  Copyright (c) 2013,2014,2015,2016
#  by Certified Scientific Software.
#  All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software ("splot") and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  Neither the name of the copyright holder nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#     * The software is provided "as is", without warranty of any   *
#     * kind, express or implied, including but not limited to the  *
#     * warranties of merchantability, fitness for a particular     *
#     * purpose and noninfringement.  In no event shall the authors *
#     * or copyright holders be liable for any claim, damages or    *
#     * other liability, whether in an action of contract, tort     *
#     * or otherwise, arising from, out of or in connection with    *
#     * the software or the use of other dealings in the software.  *
#
#******************************************************************************

version = "3.3"

def getVersion():
    global version
    short = version.split("-")[0]
    return short


def getFullVersion():
    return version

if __name__ == '__main__':
    print(getFullVersion())





#  %W%  %G% CSS
#  "pyspec" Release %R%
#
"""SpecWaitObject module

This module defines the classes for helper objects
designed for waiting specific events from Spec

Classes:
SpecWaitObject -- base class for Wait objects

Functions:
waitChannel -- wait for a channel update
waitReply -- wait for a reply
waitConnection -- wait for a connection
"""

import weakref
import time
import sys

from pyspec.utils import is_python2

import SpecEventsDispatcher as SpecEventsDispatcher
from SpecClientError import SpecClientError, SpecClientTimeoutError

def waitFunc(timeout):
  """Waiting function

  Arguments:
  timeout -- waiting time in milliseconds
  """
  try:
      from pyspec.client.SpecConnectionsManager import SpecConnectionsManager
      P = getattr(SpecConnectionsManager(), "poll")
  except AttributeError:
      time.sleep(timeout/1000.0)
      SpecEventsDispatcher.dispatch()
  else:
      P(timeout/1000.0)

class SpecWaitObject:
    """Helper class for waiting specific events from Spec"""
    def __init__(self, connection):
        """Constructor

        Arguments:
        connection -- a SpecConnection object
        """
        self.connection = weakref.ref(connection)
        self.isdisconnected = True
        self.channelWasUnregistered = False
        self.value = None

        SpecEventsDispatcher.connect(connection, 'connected', self.connected)
        SpecEventsDispatcher.connect(connection, 'disconnected', self.disconnected)

        if connection.isSpecConnected():
            self.connected()

    def connected(self):
        """Callback triggered by a 'connected' event."""
        self.isdisconnected = False

    def disconnected(self):
        """Callback triggered by a 'disconnected' event."""
        self.isdisconnected = True

    def waitReply(self, command, argsTuple, timeout = None):
        """Wait for a reply from Spec

        Arguments:
        command -- method returning a replyID to be executed on the connection object
        argsTuple -- tuple of arguments to be passed to the command
        timeout -- optional timeout (defaults to None)
        """
        connection = self.connection()

        if connection is not None:
            try:
                func = getattr(connection, command)
            except:
                return
            else:
                if callable(func):
                    func(*argsTuple)

                self.wait(timeout = timeout)

    def waitChannelUpdate(self, chanName, waitValue = None, timeout = None):
        """Wait for a channel update

        Arguments:
        chanName -- channel name
        waitValue -- particular value to wait (defaults to None, meaning any value)
        timeout -- optional timeout (defaults to None)
        """
        connection = self.connection()

        if connection is not None:
            self.channelWasUnregistered = False
            channel = connection.getChannel(chanName)

            if not channel.registered:
                self.channelWasUnregistered = True
                connection.registerChannel(chanName, self.channelUpdated) #channel.register()
            else:
                SpecEventsDispatcher.connect(channel, 'valueChanged', self.channelUpdated)

            self.wait(waitValue = waitValue, timeout = timeout)

            if self.channelWasUnregistered:
                connection.unregisterChannel(chanName) #channel.unregister()

    def waitConnection(self, timeout = None):
        """Wait for the connection to Spec being established

        Arguments:
        timeout -- optional timeout (defaults to None)

        Exceptions:
        timeout -- raise a timeout exception on timeout
        """
        connection = self.connection()

        if connection is not None:
            t = 0

            while self.isdisconnected:
                SpecEventsDispatcher.dispatch()

                t0 = time.time()
                waitFunc(10)
                t += (time.time() - t0)*1000

                if timeout is not None and t >= timeout:
                    raise SpecClientTimeoutError

    def wait(self, waitValue = None, timeout = None):
        """Block until the object's internal value gets updated

        Arguments:
        waitValue -- particular value to wait (defaults to None, meaning any value)
        timeout -- optional timeout (defaults to None)

        Exceptions:
        timeout -- raise a timeout exception on timeout
        """
        t0 = time.time()
        while not self.isdisconnected:
            waitFunc(10)

            if self.value is not None:
                if waitValue is None:
                    return

                if waitValue == self.value:
                    return
                else:
                    self.value = None

            if self.value is None:
                t = (time.time() - t0)*1000
                if timeout is not None and t >= timeout:
                    raise SpecClientTimeoutError

    def replyArrived(self, reply):
        """Callback triggered by a reply from Spec."""
        self.value = reply.get_data()

        if reply.error:
            raise SpecClientError('Server request did not complete: %s' % self.value, reply.error_code)

    def channelUpdated(self, channelValue):
        """Callback triggered by a channel update

        If channel was unregistered, we skip the first update,
        else we update our internal value
        """
        if self.channelWasUnregistered == True:
            #
            # if we were unregistered, skip first update
            #
            self.channelWasUnregistered = 2
        else:
            self.value = channelValue

def waitConnection(connection, timeout = None):
    """Wait for a connection to Spec to be established

    Arguments:
    connection -- a 'host:port' string
    timeout -- optional timeout (defaults to None)
    """
    if isinstance(connection, str) or (is_python2() and isinstance(connection, unicode)):
        from SpecConnection import SpecConnection 
        connection = SpecConnection( str(connection) )

    w = SpecWaitObject(connection)
    w.waitConnection(timeout = timeout)

def waitChannelUpdate(chanName, connection, waitValue = None, timeout = None):
    """Wait for a channel to be updated

    Arguments:
    chanName -- channel name (e.g 'var/toto')
    connection -- a 'host:port' string
    waitValue -- value to wait (defaults to None)
    timeout -- optional timeout (defaults to None)
    """
    if isinstance(connection, str) or (is_python2() and isinstance(connection, unicode)):
      connection = str(connection)
      from pyspec.client.SpecConnectionsManager import SpecConnectionsManager
      connection = SpecConnectionsManager().getConnection(connection)
      waitConnection(connection, timeout = timeout)

    w = SpecWaitObject(connection)
    w.waitChannelUpdate(chanName, waitValue = waitValue, timeout = timeout)

    return w.value


def waitReply(connection, command, argsTuple, timeout = None):
    """Wait for a reply from a remote Spec server

    Arguments:
    connection -- a 'host:port' string
    command -- command to execute
    argsTuple -- tuple of arguments for the command
    timeout -- optional timeout (defaults to None)
    """
    if isinstance(connection, str) or (is_python2() and isinstance(connection, unicode)):
      connection = str(connection)
      from pyspec.client.SpecConnectionsManager import SpecConnectionsManager
      connection = SpecConnectionsManager().getConnection(connection)
      waitConnection(connection, timeout = timeout)

    w = SpecWaitObject(connection)
    w.waitReply(command, argsTuple, timeout=timeout)

    return w.value




#  %W%  %G% CSS
#  "pyspec" Release %R%
#
"""SpecCounter module

This module defines the classes for counter objects

Classes:
SpecCounter -- class representing a counter in Spec
SpecCounterA -- class representing a counter in Spec, to be used with a GUI
"""

__author__ = 'Matias Guijarro'
__version__ = '1.0'

import SpecConnectionsManager as SpecConnectionsManager
import SpecEventsDispatcher as SpecEventsDispatcher
import SpecWaitObject as SpecWaitObject

(COUNTING) = (3)
(UNKNOWN, SCALER, TIMER, MONITOR) = (0, 1,2,3)

class SpecCounter:
    """SpecCounter class"""
    def __init__(self, specName = None, specVersion = None, timeout = None):
        """Constructor

        Keyword arguments:
        specName -- the name of the counter in Spec (defaults to None)
        specVersion -- 'host:port' string representing a Spec server to connect to (defaults to None)
        timeout -- optional timeout for connection (defaults to None)
        """
        self.channelName = ''
        self.connection = None
        self.type = UNKNOWN

        if specName is not None and specVersion is not None:
            self.connectToSpec(specName, specVersion, timeout)
        else:
            self.specName = None
            self.specVersion = None


    def connectToSpec(self, specName, specVersion, timeout = None):
        """Connect to a remote Spec

        Connect to Spec

        Arguments:
        specName -- name of the counter in Spec
        specVersion -- 'host:port' string representing a Spec server to connect to
        timeout -- optional timeout for connection (defaults to None)
        """
        self.specName = specName
        self.specVersion = specVersion

        self.connection = SpecConnectionsManager.SpecConnectionsManager().getConnection(specVersion)

        w = SpecWaitObject.SpecWaitObject(self.connection)
        w.waitConnection(timeout)

        c = self.connection.getChannel('var/%s' % self.specName)
        index = c.read()
        if index == 0:
            self.type = TIMER
        elif index == 1:
            self.type = MONITOR
        else:
            self.type = SCALER


    def count(self, time):
        """Count up to a certain time or monitor count

        Arguments:
        time -- count time
        """
        if self.connection is not None:
            c1 = self.connection.getChannel('scaler/.all./count')
            c2 = self.connection.getChannel('scaler/%s/value' % self.specName)

            if self.type == MONITOR:
                time = -time

            c1.write(time)

            w = SpecWaitObject.SpecWaitObject(self.connection)
            w.waitChannelUpdate('scaler/.all./count', waitValue = 0)

            return c2.read()


    def getValue(self):
        """Return current counter value."""
        if self.connection is not None:
            c = self.connection.getChannel('scaler/%s/value' % self.specName)

            return c.read()

















#  %W%  %G% CSS
#  "pyspec" Release %R%
#
"""SpecCommand module
.
This module defines the classes Spec command
objects

Classes:
BaseSpecCommand
SpecCommand
SpecCommandA
"""

import sys
import time

from pyspec.css_logger import log
from pyspec.utils import is_python2

import SpecEventsDispatcher as SpecEventsDispatcher
from SpecClientError import SpecClientTimeoutError

class SpecCommand(object):
    """Base class for SpecCommand objects"""
    synchronous = True

    def __init__(self, conn, command, callbacks = {}, timeout=5):

        self.command = command
        self._conn = conn
        self.timeout = timeout

        self.reply_pending = False
        self.retvalue = None

        self.specapp = self._conn.get_specname()

        # save callbacks
        self.__callback = None
        self.__error_callback = None

        self.__callbacks = {
            'connected': None,
            'disconnected': None,
            'statusChanged': None,
        }
       
        for cb_name in iter(self.__callbacks.keys()):
            if callable(callbacks.get(cb_name)):
                self.__callbacks[cb_name] = SpecEventsDispatcher.callableObjectRef(callbacks[cb_name])

        if self.synchronous:
            self._conn.wait_connected()
        else:
            SpecEventsDispatcher.connect(self._conn, 'connected', self._connected)
            SpecEventsDispatcher.connect(self._conn, 'disconnected', self._disconnected)

            if self._conn.is_connected():
                self._connected()

    def is_connected(self):
        return self._conn is not None and self._conn.is_connected()

    def __repr__(self):
        return '<SpecCommand object, command=%s>' % self.command or ''

    def __call__(self, *args, **kwargs):
        if self.command is None:
            return

        if not self.is_connected():
            return

        self.__callback = kwargs.get("callback", None)
        self.__error_callback = kwargs.get("error_callback", None)

        if self._conn.server_version < 3:
            func = False

            if 'function' in kwargs:
                func = kwargs['function']

            #convert args list to string args list
            #it is much more convenient using .call('psvo', 12) than .call('psvo', '12')
            #a possible problem will be seen in Spec
            args = map(repr, args)

            if func:
                # macro function
                command = self.command + '(' + ','.join(args) + ')'
            else:
                # macro
                command = self.command + ' ' + ' '.join(args)
        else:
            # Spec knows
            command = [self.command] + list(args)

        self.reply_pending = True
        self.retvalue = None

        return self.executeCommand(command)

    def executeCommand(self, command):
        if self._conn.server_version < 3:
            conn_cmd = 'send_msg_cmd_with_return'
        else:
            if isinstance(command,str):
                conn_cmd = self._conn.send_msg_cmd_with_return
            else:
                conn_cmd = self._conn.send_msg_func_with_return

        reply_id = conn_cmd(command)

        if self.synchronous:
            self.wait()
            return self.retvalue
        else:
            return reply_id

    def replyArrived(self, reply):
        self.reply_pending = False
        self.retvalue = reply.get_data()

        if reply.error:
            if callable(self.__error_callback):
                try:
                    self.__error_callback(reply.error)
                except:
                    log.exception("Error while calling error callback (command=%s,spec version=%s)", self.command, self.specapp)
                self.__error_callback = None
        else:
            if callable(self.__callback):
                try:
                    self.__callback(reply.data)
                except:
                    log.exception("Error while calling reply callback (command=%s,spec version=%s)", self.command, self.specapp)
                self.__callback = None

    def wait(self):
        start_wait = time.time()
        while self.reply_pending:
            elapsed = time.time() - start_wait
            if elapsed > self.timeout:
                raise SpecClientTimeoutError("timeout waiting for command execution")
            time.sleep(0.02)

    def abort(self):
        self._conn.abort()

    def connected(self):
        pass

    def _connected(self):
        self._conn.registerChannel("status/ready", self._statusChanged)
        self._conn.send_msg_hello()        

        try:
            cb_ref = self.__callbacks.get("connected")
            if cb_ref is not None:
                cb = cb_ref()
                if cb is not None:
                    cb()
        finally:
            self.connected()

    def _disconnected(self):
        try:
            cb_ref = self.__callbacks.get("disconnected")
            if cb_ref is not None:
                cb = cb_ref()
                if cb is not None:
                    cb()
        finally:
            self.disconnected()

    def disconnected(self):
        pass

    def _statusChanged(self, ready):
        try:
            cb_ref = self.__callbacks.get("statusChanged")
            if cb_ref is not None:
                cb = cb_ref()
                if cb is not None:
                   cb(ready)
        finally:
            self.statusChanged(ready)
    
    def statusChanged(self, ready):
        pass

class SpecCommandA(SpecCommand):
    """SpecCommandA is the asynchronous version of SpecCommand.
    It allows custom waiting by subclassing."""
    synchronous = False




#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#$Id: SpecConnectionsManager.py,v 1.13 2005/09/27 13:54:58 guijarro Exp $
"""Module for managing connections to Spec

The SpecConnectionsManager module provides facilities to get
a connection to Spec. It can run a thread for 'asynchronous'
polling of socket events. It prevents from having more than
one connection to the same Spec server at the same time, and
automatically reconnects lost connections.


Classes :
  _ThreadedSpecConnectionsManager
  _SpecConnectionsManager
"""

__author__ = 'Matias Guijarro'
__version__ = '1.1'

import atexit
import threading
import time
import weakref
import asyncore
import sys
import gc

import SpecConnection as SpecConnection
import SpecEventsDispatcher as SpecEventsDispatcher

from pyspec.css_logger import log

_SpecConnectionsManagerInstance = None #keep a reference to the Singleton instance

def SpecConnectionsManager(pollingThread = True, also_dispatch_events=False):
    """Return the Singleton Spec connections manager instance"""
    global _SpecConnectionsManagerInstance

    if _SpecConnectionsManagerInstance is None:
        if pollingThread:
            _SpecConnectionsManagerInstance = _ThreadedSpecConnectionsManager(also_dispatch_events)

            def _endSpecConnectionsManager():
                global _SpecConnectionsManagerInstance

                if _SpecConnectionsManagerInstance is not None:
                    _SpecConnectionsManagerInstance.stop()
                    _SpecConnectionsManagerInstance = None

            # register _endSpecConnectionsManager() to be called on Python interpreter exit
            atexit.register(_endSpecConnectionsManager)
        else:
            _SpecConnectionsManagerInstance = _SpecConnectionsManager()

    return _SpecConnectionsManagerInstance


class _ThreadedSpecConnectionsManager(threading.Thread):
    """Class for managing connections to Spec

    Polling of asynchronous socket events is delegated to a separate thread

    Warning: should never be instanciated directly ; use the module level SpecConnectionsManager()
    function instead.
    """
    def __init__(self, dispatch_events):
        """Constructor"""
        threading.Thread.__init__(self)

        self.lock = threading.Lock()
        self.connections = {}
        self.connectionDispatchers = {}
        self.stopEvent = threading.Event()
        self.__started = False
        self.doEventsDispatching = dispatch_events
        self.setDaemon(True)


    def run(self):
        """Override Thread.run() ; define behaviour for the connections manager thread

        For each SpecConnection object in the connections dictionary, try to make
        a connection. If the connection is already established, nothing occurs.
        Poll the asyncore dispatchers for processing socket events.
        """
        self.__started = True

        while not self.stopEvent.isSet():
            self.lock.acquire()
            try:
                connection_dispatcher_keys = self.connectionDispatchers.keys()
                for k in connection_dispatcher_keys:
                  connection = self.connectionDispatchers.get(k)
                  if connection is not None:
                    connection.makeConnection()

                if self.stopEvent.isSet():
                    break
            finally:
                self.lock.release()

            if len(self.connectionDispatchers) > 0:
                asyncore.loop(0.01, False, None, 1)
                if self.doEventsDispatching:
                  SpecEventsDispatcher.dispatch()
            else:
                time.sleep(0.01)

        asyncore.loop(0.01, False, None, 1)


    def stop(self):
        """Stop the connections manager thread and dereferences all connections"""
        self.stopEvent.set()

        try:
           self.join()
        except RuntimeError:
           # forget this for now as this code should dissapera
           pass

        self.__started = False
        self.connections = {}


    def getConnection(self, specVersion):
        """Return a SpecConnection object

        Arguments:
        specVersion -- a string in the 'host:port' form
        """
        gc.collect()

        try:
            con = self.connections[specVersion]()
        except KeyError:
            con = SpecConnection.SpecConnection(specVersion)

            def removeConnection(ref, connectionName = specVersion):
                self.closeConnection(connectionName)

            self.connections[specVersion] = weakref.ref(con, removeConnection)

            self.lock.acquire()
            try:
                self.connectionDispatchers[specVersion] = con.dispatcher
            finally:
                self.lock.release()

        if not self.__started:
            self.start()
            self.__started = True

        return con


    def closeConnection(self, specVersion):
        self.lock.acquire()
        try:
            self.connectionDispatchers[specVersion].handle_close()

            del self.connectionDispatchers[specVersion]
            del self.connections[specVersion]
        finally:
            self.lock.release()


    def closeConnections(self):
        for connectionName in self.connectionDispatchers.keys():
            self.closeConnection(connectionName)


class _SpecConnectionsManager:
    """Class for managing connections to Spec

    The poll() method should be called inside a GUI loop during idle time.
    Unlike the threaded class, the poll method will also dispatch SpecClient events

    Warning: should never be instanciated directly ; use the module level SpecConnectionsManager()
    function instead.
    """
    def __init__(self):
        """Constructor"""
        self.connections = weakref.WeakValueDictionary()

    def getFdDispatchersDict(self):
        connection_keys = self.connections.keys()
        connection_dispatchers = {}
        for k in connection_keys:
          connection = self.connections.get(k) #()
          if connection is not None:
            condis = connection.dispatcher  
            condis.makeConnection()
            if condis.socket is not None:
                try:
                   connection_dispatchers[condis.socket.fileno()]=condis
                except:
                   # BAD FILE DESCRIPTOR?
                   continue
        return connection_dispatchers
       
    def poll(self, timeout=0.01):
        """Poll the asynchronous socket connections and dispatch incomming events"""
        fd = self.getFdDispatchersDict()
        if -1 not in fd:
            asyncore.loop(timeout, False, fd, 1)
        else:
            pass

        SpecEventsDispatcher.dispatch()


    def stop(self):
        """Stop the connections manager thread and dereferences all connections"""
        self.connections = weakref.WeakValueDictionary()


    def getConnection(self, specVersion):
        """Return a SpecConnection object

        Arguments:
        specVersion -- a string in the 'host:port' form
        """
        con = self.connections.get(specVersion)
        if con is None: # or con() is None:
            con = SpecConnection.SpecConnection(specVersion)
            self.connections[specVersion] = con
        #else:
        #    con = con()

        return con


    def closeConnection(self, specVersion):
        try:
            del self.connections[specVersion]
        except:
            pass


    def closeConnections(self):
        for connectionName in self.connections.keys():
            self.closeConnection(connectionName)




#******************************************************************************
#
#  %W%  %G% CSS
#
#  "pyspec" Release %R%
#
#  Copyright (c) 2016,2017,2018,2020
#  by Certified Scientific Software.
#  All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software ("pyspec") and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  Neither the name of the copyright holder nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#     * The software is provided "as is", without warranty of any   *
#     * kind, express or implied, including but not limited to the  *
#     * warranties of merchantability, fitness for a particular     *
#     * purpose and noninfringement.  In no event shall the authors *
#     * or copyright holders be liable for any claim, damages or    *
#     * other liability, whether in an action of contract, tort     *
#     * or otherwise, arising from, out of or in connection with    *
#     * the software or the use of other dealings in the software.  *
#
#******************************************************************************

from graphics_rc import g_rc

try:
    import matplotlib

    if g_rc.qt_variant == "PySide":
        matplotlib.use("Qt4Agg")
        from matplotlib import rcParams
        rcParams["backend.qt4"] = "PySide"
        import matplotlib.backends.backend_qt4agg as mpl_backend
    elif g_rc.qt_variant == "PySide2":
        matplotlib.use("Qt5Agg")
        #from matplotlib import rcParams
        #rcParams["backend.qt5"] = "PySide2"
        import matplotlib.backends.backend_qt5agg as mpl_backend
    elif g_rc.qt_variant in ["PyQt5", "PySide2"]:
        matplotlib.use("Qt5Agg")
        import matplotlib.backends.backend_qt5agg as mpl_backend
    else:
        matplotlib.use("Qt4Agg")
        import matplotlib.backends.backend_qt4agg as mpl_backend

    g_rc.mpl_imported = True
    g_rc.mpl_version = matplotlib.__version__
    version_parts = g_rc.mpl_version.split(".")
    mpl_major, mpl_minor = version_parts[:2]
    g_rc.mpl_version_no = [int(mpl_major), int(mpl_minor), 0]

    if len(version_parts) > 2:
       try:
           import re
           rel = version_parts[2]
           m = re.search("(?P<rel>\d+)", rel )
           if m:
               mrel = int(m.group('rel'))
               g_rc.mpl_version_no[2] = mrel
       except:
          pass

    g_rc.graph_variant = "matplotlib"

except:
    pass




#!/usr/bin/env python
#
#******************************************************************************
#
#  %W%  %G% CSS
#
#  "pyspec" Release %R%
#
#  Copyright (c) 2020
#  by Certified Scientific Software.
#  All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software ("pyspec") and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  Neither the name of the copyright holder nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#     * The software is provided "as is", without warranty of any   *
#     * kind, express or implied, including but not limited to the  *
#     * warranties of merchantability, fitness for a particular     *
#     * purpose and noninfringement.  In no event shall the authors *
#     * or copyright holders be liable for any claim, damages or    *
#     * other liability, whether in an action of contract, tort     *
#     * or otherwise, arising from, out of or in connection with    *
#     * the software or the use of other dealings in the software.  *
#
#******************************************************************************

import os
import sys
import copy

SPECD='/usr/local/lib/spec.d'
specd = os.environ.get('SPECD', SPECD)
sys.path.append( specd )

import numpy as np
from PIL import Image

from pyspec.graphics.QVariant import *
from pyspec.client.SpecCommand import SpecCommand
from pyspec.file import tiff

FigureCanvas = mpl_backend.FigureCanvasQTAgg
NavigationToolbar = mpl_backend.NavigationToolbar2QT

from matplotlib.figure import Figure
from matplotlib.colors import LogNorm
import matplotlib.patches as patches
import matplotlib.colorbar as colorbar

from matplotlib import rcParams
rcParams.update({'font.size': 12})

class RoiNavigationBar(NavigationToolbar):
    def __init__(self, plt_canvas,parent):
        NavigationToolbar.__init__(self, plt_canvas, parent)
        self._active = "ZOOM"
        self.zoom()

        next=None

        for c in self.findChildren(QToolButton):
            if next is None:
                next=c

            # Don't want to see subplots and customize
            if str(c.text()) in ('Subplots','Customize'):
                c.defaultAction().setVisible(False)
                continue

    def press_pan(self,event):
        if event.button == 3:
           self._button_pressed = None
           return

        NavigationToolbar.press_pan(self, event)

    def press_zoom(self,event):
        if event.button == 3:
           self._button_pressed = None
           return

        NavigationToolbar.press_zoom(self, event)

class RoiSelector(QWidget):

    image_dir = "/data/images"

    rows = 514
    cols = 1030

    tiffoffset = 4096

    help_text = "Navigate and ZOOM on image with help of LEFT button and Navigation Bar\n" \
           "Select ROI by clicking and dragging with RIGHT mouse button\n" \
           "Then click apply to send to spec"

    def __init__(self,*args):

        super(RoiSelector,self).__init__(*args)

        self.spec_app = None
        self.roiname = None
        self.xbeg = self.xend = self.ybeg = self.yend = None
        self.log_set = True
        self.data = None

        #self.load_data = self.loadDataAlbula
        #self.load_data = self.loadDataNumpy
        self.load_data = self.loadDataTiff

        #  other possible methods to load data
        #self.load_data = self.loadDataSpec
        #self.load_data = self.loadDataArray

        thelayout = QVBoxLayout()
        self.setLayout(thelayout)

        self.counter_layout = QHBoxLayout()
        self.color_layout = QHBoxLayout()
        self.bottom_layout = QHBoxLayout()

        self.selecting = False
        self.last_rect = None

        self.minval = None
        self.maxval = None

        # create a figure, add an axes area and make space for a colorbar
        self.fig = Figure(figsize=(300,200), dpi=100)
        self.axes = self.fig.add_subplot(111)

        self.caxes, kw = colorbar.make_axes(self.axes, location="right", pad=0.05, fraction=0.046)

        # set all font sizes to 9
        lblsz = 9
        self.axes.tick_params(labelsize=lblsz)    
        self.axes.tick_params(labelsize=lblsz)    
        self.caxes.tick_params(labelsize=lblsz)
        self.caxes.tick_params(labelsize=lblsz)

        # create a qt widget that contains the figure
        self.figcanvas = FigureCanvas(self.fig)
        self.figcanvas.setParent(self)
        self.figcanvas.setFocusPolicy(Qt.StrongFocus)
        self.figcanvas.setFocus()

        self.navbar = RoiNavigationBar(self.figcanvas, self)

        self.top_text = QLabel()
        self.top_text.setText(self.help_text)
        self.top_text.setStyleSheet("font-style: italic;")

        #
        counters_label = QLabel("Setting ROI for counter:")
        self.counter_label = QLabel()
        self.counter_combo = QComboBox()

        # add button and put everything in qt layout
        self.coordlab = QLabel()

        selection_label = QLabel("Selection:")
        self.selection_wid = QLineEdit()
        self.selection_wid.setText("----")
        self.selection_wid.setReadOnly(True)

        colormap_label = QLabel("Colormap:")
        self.colormap_combo = QComboBox()
        self.colormap_combo.insertItems(0,["hot","nipy_spectral","gist_rainbow","prism", "gist_ncar","binary","gray"])
        self.colormap_combo.currentIndexChanged.connect(self.colormap_changed)
        self.colormap = "hot"

        self.log_scale_rb = QRadioButton("Log")
        self.log_scale_rb.toggled.connect(self.log_scale_set)
        self.log_scale_rb.setChecked(self.log_set)

        minval_label = QLabel("Min:")
        self.minval_le = QLineEdit()
        self.minval_le.returnPressed.connect(self.minval_changed)
        maxval_label = QLabel("Max:")
        self.maxval_le = QLineEdit()
        self.maxval_le.returnPressed.connect(self.maxval_changed)

        change_but = QPushButton("Change Image")
        change_but.clicked.connect(self.load_image)

        apply_but = QPushButton("Apply")
        apply_but.clicked.connect(self.send_roi_to_spec)

        close_but = QPushButton("Close")
        close_but.clicked.connect(self.close_me)
 
 
        thelayout.addWidget(self.navbar)
        thelayout.addWidget(self.top_text)
        thelayout.addWidget(self.figcanvas)
        thelayout.addWidget(self.coordlab)

        thelayout.addLayout(self.counter_layout)
        thelayout.addLayout(self.color_layout)
        thelayout.addLayout(self.bottom_layout)

        self.counter_layout.addWidget(counters_label)
        self.counter_layout.addWidget(self.counter_label)
        self.counter_layout.addWidget(self.counter_combo)
        self.counter_layout.addWidget(selection_label)
        self.counter_layout.addWidget(self.selection_wid)

        self.color_layout.addWidget(self.log_scale_rb)
        self.color_layout.addWidget(colormap_label)
        self.color_layout.addWidget(self.colormap_combo)
        self.color_layout.addWidget(minval_label)
        self.color_layout.addWidget(self.minval_le)
        self.color_layout.addWidget(maxval_label)
        self.color_layout.addWidget(self.maxval_le)

        self.bottom_layout.addWidget(apply_but)
        self.bottom_layout.addWidget(change_but)
        self.bottom_layout.addWidget(close_but)
      
        self.counter_combo.currentIndexChanged.connect(self.counter_selected)

        # register for matplotlib events
        self.figcanvas.mpl_connect("button_press_event", self.start_selection)
        self.figcanvas.mpl_connect("button_release_event", self.end_selection)
        self.figcanvas.mpl_connect("motion_notify_event", self.mouse_moved)

    def set_spec(self, specname):
        self.spec_app = specname

        try:
            spec_cmd = SpecCommand(self.getcounters_macro, "localhost:%s" % self.spec_app)
            self.counters = spec_cmd()
        except:
            import traceback
            traceback.print_exc()
            self.counters = None

        if self.counters:
            self.cnts = self.counters.split(",")
            if len(self.cnts) > 1:
                self.counter_label.hide()
                self.counter_combo.insertItems(0,self.cnts)
            else:
                self.counter_combo.hide()

            self.set_roiname(self.cnts[0])

    def set_detector(self, prefix):
        self.setroi_macro = "%s_setroi" % prefix
        self.getroi_macro = "%s_getroi" % prefix
        self.getcounters_macro = "%s_getcounters" % prefix

    def set_roiname(self, roiname):
        if roiname is None:
            print("Ignoring set_roiname")
            return

        self.roiname = roiname
        if len(self.cnts) == 1:
            self.counter_label.setText(self.roiname)
        spec_cmd = SpecCommand(self.getroi_macro, "localhost:%s" % self.spec_app)
        roivals = spec_cmd(self.roiname)
        self.set_coords(roivals)

    def counter_selected(self,index):
        selected = str(self.counter_combo.currentText())
        self.set_roiname(selected)

    def colormap_changed(self,index):
        self.colormap = str(self.colormap_combo.currentText())
        self.show_image()

    def log_scale_set(self,flag):
        self.log_set = flag
        if self.data is None:
             return
        self.show_image()

    def minval_changed(self):
        try:
            self.minval = int(str(self.minval_le.text()))
            self.show_image()
        except:
            pass

    def maxval_changed(self):
        try:
            self.maxval = int(str(self.maxval_le.text()))
            self.show_image()
        except:
            import traceback
            traceback.print_exc()
            pass

    def set_coords(self,coords):
        self.xbeg = int(coords["begx"])
        self.ybeg = int(coords["begy"])
        self.xend = int(coords["endx"])
        self.yend = int(coords["endy"])

        selection_text = "(%s,%s) (%s,%s)" % (self.xbeg, self.ybeg, self.xend, self.yend)
        self.selection_wid.setText(selection_text)

        self.show_roi_rect()

    def show_roi_rect(self):
        if None in [self.xend, self.yend, self.xbeg, self.ybeg]:
             return
        self.width  = self.xend - self.xbeg
        self.height  = self.yend - self.ybeg

        x0,y0,w,h = self.xbeg,self.ybeg, self.width,self.height

        if self.last_rect:
            try:
                self.last_rect.remove() 
            except:
                #import traceback
                #traceback.print_exc()
                print("last_rect exists but it cannot be removed")

        self.last_rect = self.axes.add_patch(patches.Rectangle( (x0, y0), w, h, color="white", fill=False) )
        self.figcanvas.draw()

    def start_selection(self,ev):
        if ev.inaxes == self.axes and ev.button == 3:
            self.selecting = True
            self.xbeg, self.ybeg = int(round(ev.xdata)), int(round(ev.ydata))
            if self.last_rect:
                try:
                    self.last_rect.remove() 
                except:
                    print("last_rect exists but it cannot be removed")
                finally:
                    self.last_rect = None

    def mouse_moved(self,ev):
        # if selecting draw rectangle while moving
        # show position if on canvas
        if ev.inaxes == self.axes:
            x = int(round(ev.xdata))
            y = int(round(ev.ydata))
            z = self.img_data[y][x]
            pos_str = "x=%3.2f, y=%3.2f, value=%3.2f" % (x,y,z)
            self.coordlab.setText(pos_str)

            if self.selecting:
                if None in [x,y]:
                    return
                self.xend, self.yend = x,y
                self.show_roi_rect()

    def end_selection(self,ev):
        if self.selecting:
            # self.xend, self.yend = int(round(ev.xdata)), int(round(ev.ydata))
            self.selecting = False

            if self.xbeg > self.xend:
               self.xbeg, self.xend = self.xend, self.xbeg
            if self.ybeg > self.yend:
               self.ybeg, self.yend = self.yend, self.ybeg

            selection_text = "(%s,%s) (%s,%s)" % (self.xbeg, self.ybeg, self.xend, self.yend)
            self.selection_wid.setText(selection_text)

    def send_roi_to_spec(self):

        if None in [self.roiname, self.xbeg, self.xend, self.ybeg, self.yend]:
             print("Please select a ROI")
             return
             
        spec_cmd = SpecCommand(self.setroi_macro, "localhost:%s" % self.spec_app)
        try:
            spec_cmd(self.roiname, self.xbeg, self.ybeg, self.xend, self.yend)
        except:
            print("Failed sending command to spec")

        #self.close_me()

    def close_me(self):
        sys.exit(0)

    def load_image(self):
        self.last_rect = None 
        self.load_data()
        self.minval = self.data.min()
        self.maxval = self.data.max()
        self.show_image()

    def show_image(self):

        self.axes.clear() 
        
        self.img_data = copy.copy(self.data)

        if self.maxval is not None:
            # apply max threshold
            max_idx = self.img_data > self.maxval
            self.img_data[max_idx] = self.maxval
           
        if self.minval is not None:
            # apply max threshold
            min_idx = self.img_data < self.minval
            self.img_data[min_idx] = self.minval
           
        self.minval_le.setText(str(self.minval))
        self.maxval_le.setText(str(self.maxval))

        width, height = self.img_data.shape 
        y,x = np.mgrid[0:width,0:height]

        if self.log_set:
            # make sure there is no neg values.
            # with matplotlib > 2.0 this is done with SymLogNorm and no data manipulation
            zmin = self.img_data.min()
            zmax = self.img_data.max()
            if zmin <= 0 :
                self.img_data[:] = self.img_data[:] - (zmin -2 )
                zmin = self.img_data.min()

            im=self.axes.pcolormesh(x,y,self.img_data, norm=LogNorm(vmin=zmin, vmax=zmax))
        else:
            im=self.axes.pcolormesh(x,y,self.img_data)
        self.axes.axis([x.min(), x.max(), y.min(), y.max()])

        #im=self.axes.imshow(self.img_data, interpolation='nearest')
        ##self.axes.invert_yaxis()

        self.fig.colorbar(im,cax=self.caxes,orientation="vertical")
        im.set_cmap(self.colormap)
        self.axes.set_aspect("auto")
        self.show_roi_rect()
        self.figcanvas.draw()

        return

    def loadDataSpec(self):
        import datashm
        self.data = datashm.getdata(self.spec_app, "dat")

    def loadDataNumpy(self):
        print("Load data numpy")
        filename = self.get_filename()
        im = Image.open(filename)
        self.data = np.array(im)
        print("image loaded with shape %s- sum = %s" % (self.data.shape, self.data.sum()))

    def loadDataTiff(self):
        filename = self.get_filename()
        im = tiff.TiffFile(filename)
        i0 = im.series[0]
        self.data = i0.asarray()
        print("image loaded with shape %s- sum = %s" % (self.data.shape, self.data.sum()))

    def loadDataArray(self):
        import array
        filename = self.get_filename()
        fd = open(filename,"rb")
        fd.seek(self.tiffoffset)  # skip the header
        arr = array.array("I")
        arr.fromfile(fd,self.rows*self.cols)
        self.data = []
        for i in range(self.rows):
            self.data.append( arr[self.cols*i:self.cols*(i+1)].tolist() )

        return self.data

    def loadDataAlbula(self):
        sys.path.append("/home/specadm/spec_dist/dectris/albula/3.2/python")
        import dectris.albula

        filename = self.get_filename()

        if dectris.albula.versionMajor() == 3:
            series = dectris.albula.DImageSeries()
            series.open(str(filename))
            first = series.first()
            img0 = series[first]
        else:
            container = dectris.albula.DImageIntContainer()
            container.openSeries(filename)
            first = container.first()
            img0 = container[first]

        self.data = img0.data()

    def get_filename(self):
        answ = QFileDialog.getOpenFileName(self, "Open Image", self.image_dir, "*");
        print("answer is %s" % str(answ))
        if isinstance(answ, str):
            filename = str(answ)
        else:
            filename = answ[0]
        print("filename is %s" % str(filename))
        return filename


def printUsage():
    print("Usage: %s spec_app detector" % sys.argv[0])

def main():
    import sys

    nb_args = len(sys.argv)
    
    if nb_args != 3:
        print("Wrong usage")
        printUsage() 
        sys.exit(0)

    spec = sys.argv[1]
    detector = sys.argv[2]

    app = QApplication([])
    win = QMainWindow()
    wid = RoiSelector()


    win.setCentralWidget(wid)
    win.show()
    win.resize( 500, 700 )
    wid.set_detector(detector)
    wid.set_spec(spec)
    wid.load_image()

    try:
        sys.exit(app.exec_())
    except BaseException as e:
        import traceback
        print( traceback.print_exc() )
        print( str(e) )

if __name__ == '__main__':
    main()



#!/usr/bin/env python

#******************************************************************************
#
#  %W%  %G%0 CSS
#
#  "pyspec" Release %R%
#
#  Copyright (c) 2020
#  by Certified Scientific Software.
#  All rights reserved.
#
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software ("pyspec") and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#
#  Neither the name of the copyright holder nor the names of its contributors
#  may be used to endorse or promote products derived from this software
#  without specific prior written permission.
#
#     * The software is provided "as is", without warranty of any   *
#     * kind, express or implied, including but not limited to the  *
#     * warranties of merchantability, fitness for a particular     *
#     * purpose and noninfringement.  In no event shall the authors *
#     * or copyright holders be liable for any claim, damages or    *
#     * other liability, whether in an action of contract, tort     *
#     * or otherwise, arising from, out of or in connection with    *
#     * the software or the use of other dealings in the software.  *
#
#******************************************************************************


import os

with open('README.rst') as dfd:
    long_description = dfd.read()

#try:
#from setuptools import setup, Extension, find_packages
#    packages = find_packages(exclude=['python/graphics'])
#except ImportError:
    # for python2 compatability
from distutils.core import setup, Extension
packages=['pyspec', 
        'pyspec/tools', 
        'pyspec/graphics',
        'pyspec/hardware',
        'pyspec/file', 
        'pyspec/client', 
        ]

install_requires = ['numpy']

datashm_dir = 'pyspec/datashm'
datashm_sources = ['datashm_py.c', 'sps.c']
datashm_sources = [os.path.join(datashm_dir, c_file) for c_file in datashm_sources]

datashm_ext = Extension('pyspec/datashm',
                    include_dirs = [datashm_dir],
                    sources = datashm_sources,)

try:
    from VERSION import getVersion
    __version__ = getVersion()
except:
    __version__ = '1.1.3'

setup(name='certif_pyspec',
	version=__version__,
	description='Python SPEC modules and tools',
        download_url='https://pypi.org/projects/certif-pyspec/',
        long_description=long_description,
	author='TXOlutions',
	author_email='txo@txolutions.com',
        url='http://github.com/txolutions/pyspec',
        packages = packages,
 	ext_modules=[ datashm_ext ],
        install_requires = install_requires,
        scripts=['tools/specfile', 'tools/roi_selector']
)




#
#  %W%  %G% CSS
#  "pyspec" Release %R%
#

import sys
import os

_pyspec_dir = os.path.dirname(os.path.realpath(__file__))
sys.path.append(_pyspec_dir)

from VERSION import getVersion, getFullVersion

__version__ = getVersion()
__fullversion__ = getFullVersion()




import struct
import re
import sys

import numpy as np

re_comment=re.compile("(?P<key>\w+)=\"(?P<value>\w+)\"")
re_group=re.compile("\[(?P<groupname>.*?)\]")

class ItexImage(object):

    def __init__(self, filename=None):
        self.head_info = None
        self.comments = {}
        self.data = None

        if filename:
           self.from_file(filename)

    def from_buffer(self, buff):

        _id = struct.unpack('2c', buff[0:2])
        _fst = ""

        for _c in _id:
            _fst += _c.decode('utf-8')

        if _fst != 'IM':
            print("not an ITEX data file")
            return

        head_info = struct.unpack('5h', buff[2:12])

        self.width, self.height = head_info[1:3]
        self.xoffset, self.yoffset = head_info[3:5]

        print(self.width, self.height)

        comm_length = head_info[0] 
        comm_string = buff[64:comm_length+64].decode('utf-8')
        self.parse_comments(comm_string)

        img_size = self.width*self.height*2
        start_img = comm_length+64
        end_img = comm_length+64+img_size

        data = np.frombuffer(buff[start_img:end_img], dtype=np.uint16) 
        self.data = data.reshape(self.height, self.width)
        print(self.data.shape)

        # only if calib present (ScalingXScalingFile in comments)
        try:
            self.calib_data = struct.unpack("672f", buff[end_img:])
        except:
            self.calib_data = None

    def from_file(self, filename):
        buff = open(filename,'rb').read()
        self.from_buffer(buff)

    def get_roisum(self, bx, by, ex, ey):
        pass

    def parse_comments(self, comms):
        curr_group = ''
        cursor = 0
        while True:
            mat = re_group.search(comms, cursor)
            if mat:
                if curr_group:
                    group_block = comms[group_begin:mat.start()]
                    print(curr_group)
                    print(group_block)
                curr_group = mat.group('groupname')
                cursor = mat.end()
                group_begin = cursor
            else:
                break
        group_block = comms[group_begin:]
        print(curr_group)
        print(group_block)


def main():
    img = ItexImage(sys.argv[1])
    data = img.data
    print(type(data))
    print(data.shape)
    print(data.mean())

if __name__ == '__main__':
    main()




#  %W%  %G% CSS
#  "pyspec" Release %R%
#
#  

import sys
import os

from pyspec.css_logger import log

sys.path.append(os.path.dirname(os.path.realpath(__file__)))

__all__ = [
    'saferef.py',
    'Spec.py',
    'SpecArray.py',
    'SpecChannel.py',
    'SpecClientError.py',
    'SpecCommand.py',
    'SpecConnection.py',
    'SpecConnectionsManager.py',
    'SpecCounter.py',
    'SpecEventsDispatcher.py',
    'SpecMessage.py',
    'SpecMotor.py',
    'SpecReply.py',
    'SpecScan.py'
    'SpecServer.py',
    'SpecVariable.py',
    'SpecWaitObject.py',
    ]


from SpecConnection import SpecConnection
spec = SpecConnection

import sys

from SpecMotor import NOTINITIALIZED, UNUSABLE, READY, \
            MOVESTARTED, MOVING, ONLIMIT

import SpecMotor
import SpecCommand
import SpecVariable

def is_python2():
    return sys.version_info[0] == 2

# Use as a decorator to override __init__  for classes
#    - checks the connection parameter and, if it is a string, opens
#    - the corresponding connection
#
#    This allows user classes to use either a string to specify the spec application
#    or an already existing connection


def spec_connector(cls):
    cls_init = cls.__init__
    def connect_init(self, specapp, pname, *args, **kwargs):
        if isinstance(specapp, str) or (is_python2() and isinstance(specapp, unicode)):
             conn = spec(specapp)
        else:
             conn = specapp
        cls_init(self, conn, pname, *args, **kwargs)
    cls.__init__ = connect_init
    return cls

@spec_connector
class motor(SpecMotor.SpecMotor):
    pass

@spec_connector
class motor_async(SpecMotor.SpecMotorA):
    pass

@spec_connector
class command(SpecCommand.SpecCommand):
    pass

@spec_connector
class command_async(SpecCommand.SpecCommandA):
    pass

@spec_connector
class variable(SpecVariable.SpecVariable):
    pass

@spec_connector
class variable_async(SpecVariable.SpecVariableA):
    pass

